@@ -17,7 +17,7 @@
 
 using namespace RDKit;
 
-struct dvv_pickle_suite : python::pickle_suite {
+struct dvv_pickle_suite : rdkit_pickle_suite {
   static python::tuple getinitargs(const DiscreteValueVect& self) {
     std::string res = self.toString();
     python::object retval = python::object(
