@@ -50,7 +50,9 @@ char *Base64Encode(const unsigned char *inText, const unsigned int inLen) {
   char *res;
   int resSize;
   resSize = (4 * inLen) / 3;
-  while (resSize % 4) resSize++;
+  while (resSize % 4) {
+    resSize++;
+  }
   res = new char[resSize + 1];
   unsigned int i = 0;
   int pos = 0;
@@ -91,9 +93,15 @@ char *Base64Decode(const char *inText, unsigned int *size) {
   for (i = 0; i < 255; i++) {
     transTable[i] = 0x80;
   }
-  for (i = 'A'; i <= 'Z'; i++) transTable[i] = (unsigned char)i - 'A';
-  for (i = 'a'; i <= 'z'; i++) transTable[i] = (unsigned char)i - 'a' + 26;
-  for (i = '0'; i <= '9'; i++) transTable[i] = (unsigned char)i - '0' + 52;
+  for (i = 'A'; i <= 'Z'; i++) {
+    transTable[i] = (unsigned char)i - 'A';
+  }
+  for (i = 'a'; i <= 'z'; i++) {
+    transTable[i] = (unsigned char)i - 'a' + 26;
+  }
+  for (i = '0'; i <= '9'; i++) {
+    transTable[i] = (unsigned char)i - '0' + 52;
+  }
   transTable[static_cast<int>('+')] = 62;
   transTable[static_cast<int>('/')] = 63;
 
