@@ -22,8 +22,7 @@ namespace UFF {
 class RDKIT_FORCEFIELD_EXPORT TorsionConstraintContrib
     : public ForceFieldContrib {
  public:
-  TorsionConstraintContrib()
-       {};
+  TorsionConstraintContrib() {}
   //! Constructor
   /*!
   \param owner          pointer to the owning ForceField
@@ -46,18 +45,19 @@ class RDKIT_FORCEFIELD_EXPORT TorsionConstraintContrib
                            double minDihedralDeg, double maxDihedralDeg,
                            double forceConst);
 
-  ~TorsionConstraintContrib() {}
-  double getEnergy(double *pos) const;
+  ~TorsionConstraintContrib() override = default;
+  double getEnergy(double *pos) const override;
 
-  void getGrad(double *pos, double *grad) const;
-  virtual TorsionConstraintContrib *copy() const {
+  void getGrad(double *pos, double *grad) const override;
+  TorsionConstraintContrib *copy() const override {
     return new TorsionConstraintContrib(*this);
-  };
+  }
 
  private:
-  void setParameters(ForceField *owner, unsigned int idx1,
-    unsigned int idx2, unsigned int idx3, unsigned int idx4,
-    double minDihedralDeg, double maxDihedralDeg, double forceConst);
+  void setParameters(ForceField *owner, unsigned int idx1, unsigned int idx2,
+                     unsigned int idx3, unsigned int idx4,
+                     double minDihedralDeg, double maxDihedralDeg,
+                     double forceConst);
   double computeDihedralTerm(double dihedral) const;
   int d_at1Idx{-1}, d_at2Idx{-1}, d_at3Idx{-1},
       d_at4Idx{-1};  //!< indices of atoms forming the dihedral angle
