@@ -343,8 +343,8 @@ bool isMoleculeReactantOfReaction(const ChemicalReaction &rxn, const ROMol &mol,
   unsigned int reactant_template_idx = 0;
   for (auto iter = rxn.beginReactantTemplates();
        iter != rxn.endReactantTemplates(); ++iter, ++reactant_template_idx) {
-    MatchVectType tvect;
-    if (SubstructMatch(mol, **iter, tvect)) {
+    auto tvect = SubstructMatch(mol, **iter, rxn.getSubstructParams());
+    if (!tvect.empty()) {
       which.push_back(reactant_template_idx);
       if (stopAtFirstMatch) {
         return true;
@@ -383,8 +383,8 @@ bool isMoleculeProductOfReaction(const ChemicalReaction &rxn, const ROMol &mol,
   unsigned int product_template_idx = 0;
   for (auto iter = rxn.beginProductTemplates();
        iter != rxn.endProductTemplates(); ++iter, ++product_template_idx) {
-    MatchVectType tvect;
-    if (SubstructMatch(mol, **iter, tvect)) {
+    auto tvect = SubstructMatch(mol, **iter, rxn.getSubstructParams());
+    if (!tvect.empty()) {
       which.push_back(product_template_idx);
       if (stopAtFirstMatch) {
         return true;
@@ -436,8 +436,8 @@ bool isMoleculeAgentOfReaction(const ChemicalReaction &rxn, const ROMol &mol,
         RDKit::Descriptors::calcAMW(mol)) {
       continue;
     }
-    MatchVectType tvect;
-    if (SubstructMatch(mol, **iter, tvect)) {
+    auto tvect = SubstructMatch(mol, **iter, rxn.getSubstructParams());
+    if (!tvect.empty()) {
       return true;
     }
   }
