@@ -1,5 +1,5 @@
 //
-//  Copyright (c) 2014-2021, Novartis Institutes for BioMedical Research Inc.
+//  Copyright (c) 2014-2023, Novartis Institutes for BioMedical Research Inc.
 //  and other RDKit contributors
 //
 //  All rights reserved.
@@ -45,6 +45,8 @@
 #include <RDGeneral/Invariant.h>
 #include <GraphMol/MonomerInfo.h>
 #include <GraphMol/Chirality.h>
+#include <GraphMol/QueryAtom.h>
+#include <GraphMol/QueryBond.h>
 
 namespace RDKit {
 typedef std::vector<MatchVectType> VectMatchVectType;
@@ -152,9 +154,9 @@ class StereoBondEndCap {
 };
 }  // namespace
 
-VectMatchVectType getReactantMatchesToTemplate(const ROMol &reactant,
-                                               const ROMol &templ,
-                                               unsigned int maxMatches) {
+VectMatchVectType getReactantMatchesToTemplate(
+    const ROMol &reactant, const ROMol &templ, unsigned int maxMatches,
+    const SubstructMatchParameters &ssparams) {
   // NOTE that we are *not* uniquifying the results.
   //   This is because we need multiple matches in reactions. For example,
   //   The ring-closure coded as:
@@ -175,7 +177,7 @@ VectMatchVectType getReactantMatchesToTemplate(const ROMol &reactant,
   //   with uniquifying their results.
   VectMatchVectType res;
 
-  SubstructMatchParameters ssps;
+  SubstructMatchParameters ssps = ssparams;
   ssps.uniquify = false;
   ssps.maxMatches = maxMatches;
   auto matchesHere = SubstructMatch(reactant, templ, ssps);
@@ -212,8 +214,9 @@ bool getReactantMatches(const MOL_SPTR_VECT &reactants,
   for (auto iter = rxn.beginReactantTemplates();
        iter != rxn.endReactantTemplates(); ++iter, i++) {
     if (matchSingleReactant == MatchAll || matchSingleReactant == i) {
-      auto matches = getReactantMatchesToTemplate(*reactants[i].get(),
-                                                  *iter->get(), maxMatches);
+      auto matches =
+          getReactantMatchesToTemplate(*reactants[i].get(), *iter->get(),
+                                       maxMatches, rxn.getSubstructParams());
       if (matches.empty()) {
         // no point continuing if we don't match one of the reactants:
         res = false;
@@ -727,16 +730,8 @@ void updateStereoBonds(RWMOL_SPTR product, const ROMol &reactant,
 void setReactantBondPropertiesToProduct(RWMOL_SPTR product,
                                         const ROMol &reactant,
                                         ReactantProductAtomMapping *mapping) {
-  ROMol::BOND_ITER_PAIR bondItP = product->getEdges();
-  while (bondItP.first != bondItP.second) {
-    Bond *pBond = (*product)[*(bondItP.first)];
-    ++bondItP.first;
-
-    if (!pBond->hasProp(common_properties::NullBond) &&
-        !pBond->hasProp(common_properties::_MolFileBondQuery)) {
-      continue;
-    }
-
+  for (unsigned int bidx = 0; bidx < product->getNumBonds(); ++bidx) {
+    auto pBond = product->getBondWithIdx(bidx);
     auto rBondBegin = mapping->prodReactAtomMap.find(pBond->getBeginAtomIdx());
     auto rBondEnd = mapping->prodReactAtomMap.find(pBond->getEndAtomIdx());
 
@@ -751,8 +746,22 @@ void setReactantBondPropertiesToProduct(RWMOL_SPTR product,
     if (!rBond) {
       continue;
     }
+    if (!pBond->hasProp(common_properties::NullBond) &&
+        !pBond->hasProp(common_properties::_MolFileBondQuery) &&
+        !rBond->hasQuery()) {
+      continue;
+    }
 
-    pBond->setBondType(rBond->getBondType());
+    if (!rBond->hasQuery()) {
+      pBond->setBondType(rBond->getBondType());
+    } else {
+      QueryBond qBond(rBond->getBondType());
+      qBond.setQuery(rBond->getQuery()->copy());
+      // replaceBond copies, so we are safe passing a pointer
+      // to a local:
+      product->replaceBond(bidx, &qBond);
+      pBond = product->getBondWithIdx(bidx);
+    }
     if (rBond->getBondType() == Bond::DOUBLE &&
         rBond->getBondDir() == Bond::EITHERDOUBLE) {
       pBond->setBondDir(Bond::EITHERDOUBLE);
@@ -864,6 +873,22 @@ void setReactantAtomPropertiesToProduct(Atom *productAtom,
   }
 }
 
+Bond *addBondToProduct(const Bond &origB, RWMol &product,
+                       unsigned int begAtomIdx, unsigned int endAtomIdx) {
+  if (!origB.hasQuery()) {
+    auto idx = product.addBond(begAtomIdx, endAtomIdx, origB.getBondType());
+    return product.getBondWithIdx(idx - 1);
+  } else {
+    QueryBond *qbond = new QueryBond(origB.getBondType());
+    qbond->setBeginAtomIdx(begAtomIdx);
+    qbond->setEndAtomIdx(endAtomIdx);
+    qbond->setQuery(origB.getQuery()->copy());
+    bool takeOwnership = true;
+    product.addBond(qbond, takeOwnership);
+    return qbond;
+  }
+}
+
 void addMissingProductBonds(const Bond &origB, RWMOL_SPTR product,
                             ReactantProductAtomMapping *mapping) {
   unsigned int begIdx = origB.getBeginAtomIdx();
@@ -874,8 +899,7 @@ void addMissingProductBonds(const Bond &origB, RWMOL_SPTR product,
   CHECK_INVARIANT(prodBeginIdxs.size() == prodEndIdxs.size(),
                   "Different number of start-end points for product bonds.");
   for (unsigned i = 0; i < prodBeginIdxs.size(); i++) {
-    product->addBond(prodBeginIdxs.at(i), prodEndIdxs.at(i),
-                     origB.getBondType());
+    addBondToProduct(origB, *product, prodBeginIdxs.at(i), prodEndIdxs.at(i));
   }
 }
 
@@ -883,7 +907,12 @@ void addMissingProductAtom(const Atom &reactAtom, unsigned reactNeighborIdx,
                            unsigned prodNeighborIdx, RWMOL_SPTR product,
                            const ROMol &reactant,
                            ReactantProductAtomMapping *mapping) {
-  auto *newAtom = new Atom(reactAtom);
+  Atom *newAtom = nullptr;
+  if (!reactAtom.hasQuery()) {
+    newAtom = new Atom(reactAtom);
+  } else {
+    newAtom = new QueryAtom(dynamic_cast<const QueryAtom &>(reactAtom));
+  }
   unsigned reactAtomIdx = reactAtom.getIdx();
   newAtom->setProp<unsigned int>(common_properties::reactantAtomIdx,
                                  reactAtomIdx);
@@ -893,14 +922,12 @@ void addMissingProductAtom(const Atom &reactAtom, unsigned reactNeighborIdx,
   // add the bonds
   const Bond *origB =
       reactant.getBondBetweenAtoms(reactNeighborIdx, reactAtomIdx);
-  unsigned int begIdx = origB->getBeginAtomIdx();
-  if (begIdx == reactNeighborIdx) {
-    product->addBond(prodNeighborIdx, productIdx, origB->getBondType());
-  } else {
-    product->addBond(productIdx, prodNeighborIdx, origB->getBondType());
+  unsigned int begIdx = productIdx;
+  unsigned int endIdx = prodNeighborIdx;
+  if (origB->getBeginAtomIdx() == reactNeighborIdx) {
+    std::swap(begIdx, endIdx);
   }
-
-  auto prodB = product->getBondBetweenAtoms(prodNeighborIdx, productIdx);
+  Bond *prodB = addBondToProduct(*origB, *product, begIdx, endIdx);
   if (origB->getBondType() == Bond::DOUBLE &&
       origB->getBondDir() == Bond::EITHERDOUBLE) {
     prodB->setBondDir(Bond::EITHERDOUBLE);
@@ -1309,7 +1336,7 @@ void addReactantAtomsAndBonds(const ChemicalReaction &rxn, RWMOL_SPTR product,
   // ---------- ---------- ---------- ---------- ---------- ----------
   // Loop over the bonds in the product and look for those that have
   // the NullBond property set. These are bonds for which no information
-  // (other than their existence) was provided in the template:
+  // (other than their existence) was provided in the template
   setReactantBondPropertiesToProduct(product, *reactant, mapping);
 
   // ---------- ---------- ---------- ---------- ---------- ----------
@@ -1333,6 +1360,14 @@ void addReactantAtomsAndBonds(const ChemicalReaction &rxn, RWMOL_SPTR product,
         Atom *productAtom = product->getAtomWithIdx(productAtomIdx);
         setReactantAtomPropertiesToProduct(productAtom, *reactantAtom,
                                            rxn.getImplicitPropertiesFlag());
+        if (reactantAtom->hasQuery()) {
+          // finally: if the reactant atom is a query we should copy over the
+          // query information. We need to replace the atom to do this
+          QueryAtom newAtom(*productAtom);
+          newAtom.setQuery(reactantAtom->getQuery()->copy());
+          // replaceAtom copies
+          product->replaceAtom(productAtomIdx, &newAtom);
+        }
       }
       // now traverse:
       addReactantNeighborsToProduct(*reactant, *reactantAtom, product,
@@ -1668,7 +1703,8 @@ bool updateBondsModifiedByReaction(
                                     pBond->getEndAtom()->getAtomMapNum())]
                               .second;
 
-            reactant.addBond(begIdx, endIdx, pBond->getBondType());
+            ReactionRunnerUtils::addBondToProduct(*pBond, reactant, begIdx,
+                                                  endIdx);
             molModified = true;
           } else if (bond->getBondType() != pBond->getBondType()) {
             bond->setBondType(pBond->getBondType());
@@ -1746,7 +1782,7 @@ bool run_Reactant(const ChemicalReaction &rxn, RWMol &reactant) {
   }
 
   auto reactantMatch = ReactionRunnerUtils::getReactantMatchesToTemplate(
-      reactant, *reactantTemplate, 1);
+      reactant, *reactantTemplate, 1, rxn.getSubstructParams());
   if (reactantMatch.empty()) {
     return false;
   }
