@@ -31,8 +31,9 @@
 #include <RDGeneral/FileParseException.h>
 #include <RDGeneral/BadFileException.h>
 #include <GraphMol/ChemTransforms/ChemTransforms.h>
+
 namespace RDKit {
-namespace {
+namespace details {
 void updateSubMolConfs(const ROMol &mol, RWMol &res,
                        boost::dynamic_bitset<> &removedAtoms) {
   // update conformer information:
@@ -52,7 +53,30 @@ void updateSubMolConfs(const ROMol &mol, RWMol &res,
     res.addConformer(newConf, false);
   }
 }
+void copyStereoGroups(const std::map<const Atom *, Atom *> &molAtomMap,
+                      const ROMol &mol, RWMol &newMol) {
+  // Copy over any stereo groups that lie in the new molecule
+  if (!mol.getStereoGroups().empty()) {
+    std::vector<StereoGroup> newStereoGroups;
+    for (auto &stereoGroup : mol.getStereoGroups()) {
+      std::vector<Atom *> newStereoAtoms;
+      for (const auto stereoGroupAtom : stereoGroup.getAtoms()) {
+        if (auto found = molAtomMap.find(stereoGroupAtom);
+            found != molAtomMap.end()) {
+          newStereoAtoms.push_back(found->second);
+        }
+      }
+      if (!newStereoAtoms.empty()) {
+        newStereoGroups.emplace_back(stereoGroup.getGroupType(),
+                                     newStereoAtoms);
+      }
+    }
+    newMol.setStereoGroups(std::move(newStereoGroups));
+  }
+}
+}  // namespace details
 
+namespace {
 struct SideChainMapping {
   int molIndex;
   int coreIndex;
@@ -60,10 +84,10 @@ struct SideChainMapping {
 
   SideChainMapping(int molIndex)
       : molIndex(molIndex), coreIndex(-1), useMatch(false) {}
+
   SideChainMapping(int molIndex, int coreIndex, bool useMatch)
       : molIndex(molIndex), coreIndex(coreIndex), useMatch(useMatch) {}
 };
-
 }  // namespace
 
 ROMol *deleteSubstructs(const ROMol &mol, const ROMol &query, bool onlyFrags,
@@ -136,7 +160,7 @@ ROMol *deleteSubstructs(const ROMol &mol, const ROMol &query, bool onlyFrags,
   res->commitBatchEdit();
   // if we removed any atoms, clear the computed properties:
   if (delList.size()) {
-    updateSubMolConfs(mol, *res, removedAtoms);
+    details::updateSubMolConfs(mol, *res, removedAtoms);
 
     res->clearComputedProps(true);
     // update our properties, but allow unhappiness:
@@ -242,7 +266,7 @@ std::vector<ROMOL_SPTR> replaceSubstructs(
     // clear conformers and computed props and do basic updates
     // on the resulting molecules, but allow unhappiness:
     for (auto &re : res) {
-      updateSubMolConfs(mol, *(RWMol *)re.get(), removedAtoms);
+      details::updateSubMolConfs(mol, *(RWMol *)re.get(), removedAtoms);
       re->clearComputedProps(true);
       re->updatePropertyCache(false);
     }
@@ -327,7 +351,7 @@ ROMol *replaceSidechains(const ROMol &mol, const ROMol &coreQuery,
   }
   newMol->commitBatchEdit();
 
-  updateSubMolConfs(mol, *newMol, removedAtoms);
+  details::updateSubMolConfs(mol, *newMol, removedAtoms);
 
   // clear computed props and do basic updates on the
   // the resulting molecule, but allow unhappiness:
@@ -486,6 +510,7 @@ ROMol *replaceCore(const ROMol &mol, const ROMol &core,
   auto *newMol = new RWMol(mol);
   std::vector<Atom *> keepList;
   std::map<Atom *, int> dummyAtomMap;
+  std::map<const Atom *, Atom *> molAtomMap;
 
   // go through the matches in query order, not target molecule
   //  order
@@ -514,6 +539,7 @@ ROMol *replaceCore(const ROMol &mol, const ROMol &core,
       Atom *sidechainAtom = newMol->getAtomWithIdx(mappingInfo.molIndex);
       // we're keeping the sidechain atoms:
       keepList.push_back(sidechainAtom);
+      molAtomMap[mol.getAtomWithIdx(mappingInfo.molIndex)] = sidechainAtom;
 
       // loop over our neighbors and see if any are in the match:
       std::list<unsigned int> nbrList;
@@ -628,6 +654,10 @@ ROMol *replaceCore(const ROMol &mol, const ROMol &core,
             if (switchIt) {
               sidechainAtom->invertChirality();
             }
+            // the neighbor count needs to be decremented, in case an additional
+            // bond to the core is present at this chiral center (unlikely!),
+            // because this bond will now be at the end of the neighbor list.
+            --whichNbr;
           }
         }
         ++whichNbr;
@@ -676,7 +706,7 @@ ROMol *replaceCore(const ROMol &mol, const ROMol &core,
   }
   newMol->commitBatchEdit();
 
-  updateSubMolConfs(mol, *newMol, removedAtoms);
+  details::updateSubMolConfs(mol, *newMol, removedAtoms);
 
   // Update any terminal dummy atom coordinates after removing atoms not in the
   // keeplist and calling updateSubMolConfs
@@ -694,24 +724,26 @@ ROMol *replaceCore(const ROMol &mol, const ROMol &core,
                                       endAtom->getIdx());
       }
     }
- 
+  }
+
   // make a guess at the position of the dummy atoms showing the attachment
   // point:
   for (auto citer = mol.beginConformers(); citer != mol.endConformers();
        ++citer) {
     Conformer &newConf = newMol->getConformer((*citer)->getId());
-    for (auto iter = dummyAtomMap.begin();
-         iter != dummyAtomMap.end(); ++iter) {
+    for (auto iter = dummyAtomMap.begin(); iter != dummyAtomMap.end(); ++iter) {
       newConf.setAtomPos(iter->first->getIdx(),
                          (*citer)->getAtomPos(iter->second));
     }
   }
-  
+
+  // copy over stereo groups
+  details::copyStereoGroups(molAtomMap, mol, *newMol);
+
   // clear computed props and do basic updates on
   // the resulting molecule, but allow unhappiness:
   newMol->clearComputedProps(true);
   newMol->updatePropertyCache(false);
- }
 
   return static_cast<ROMol *>(newMol);
 }
@@ -795,7 +827,7 @@ ROMol *MurckoDecompose(const ROMol &mol) {
   }
   res->commitBatchEdit();
 
-  updateSubMolConfs(mol, *res, removedAtoms);
+  details::updateSubMolConfs(mol, *res, removedAtoms);
   res->clearComputedProps();
 
   return (ROMol *)res;
@@ -965,6 +997,7 @@ void parseQueryDefFile(std::istream *inStream,
     queryDefs[qname] = msptr;
   }
 }
+
 void parseQueryDefFile(const std::string &filename,
                        std::map<std::string, ROMOL_SPTR> &queryDefs,
                        bool standardize, const std::string &delimiter,
@@ -979,6 +1012,7 @@ void parseQueryDefFile(const std::string &filename,
   parseQueryDefFile(&inStream, queryDefs, standardize, delimiter, comment,
                     nameColumn, smartsColumn);
 }
+
 void parseQueryDefText(const std::string &queryDefText,
                        std::map<std::string, ROMOL_SPTR> &queryDefs,
                        bool standardize, const std::string &delimiter,
