@@ -19,15 +19,18 @@
 #include <RDGeneral/FileParseException.h>
 #include "RDDepictor.h"
 #include "DepictUtils.h"
+#include "Templates.h"
 #include <GraphMol/SmilesParse/SmilesParse.h>
 #include <GraphMol/Substruct/SubstructMatch.h>
 #include <GraphMol/ChemTransforms/ChemTransforms.h>
 #include <GraphMol/MolAlign/AlignMolecules.h>
 #include <GraphMol/Conformer.h>
+#include <GraphMol/MolTransforms/MolTransforms.h>
 #include <Geometry/point.h>
 #include <Geometry/Transform3D.h>
 #include <RDGeneral/utils.h>
 #include <cstdlib>
+#include <cmath>
 
 #include <boost/tokenizer.hpp>
 typedef boost::tokenizer<boost::char_separator<char>> tokenizer;
@@ -821,13 +824,12 @@ void testGitHubIssue1073() {
     RWMol *m = SmartsToMol(smarts);
     TEST_ASSERT(m);
 
+    // compute2DCoords does ring finding internally, so there's no error
     RDDepict::compute2DCoords(*m);
 
+    // but the molecule itself is not modified
     RingInfo *ri = m->getRingInfo();
-    TEST_ASSERT(ri->isInitialized());
-    TEST_ASSERT(ri->isAtomInRingOfSize(0, 6));
-    TEST_ASSERT(ri->isAtomInRingOfSize(0, 5));
-    TEST_ASSERT(!ri->isAtomInRingOfSize(0, 9));
+    TEST_ASSERT(!ri->isInitialized());
 
     delete m;
   }
@@ -1289,6 +1291,79 @@ M  END)RES"_ctab;
     msd /= static_cast<double>(matchVect.size());
     TEST_ASSERT(msd < 1.0e-4);
   }
+
+  // test that using a reference with query atoms including H works
+  auto scaffold = R"CTAB(
+  MJ201100                      
+
+ 12 13  0  0  0  0  0  0  0  0999 V2000
+   -0.5398    0.0400    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -1.3648    0.0400    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -1.7773   -0.6745    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -1.3649   -1.3889    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -0.5399   -1.3889    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -0.1273   -0.6744    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+    0.6976   -0.6744    0.0000 L   0  0  0  0  0  0  0  0  0  0  0  0
+   -1.9167    0.6531    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -2.6704    0.3176    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -2.5842   -0.5028    0.0000 N   0  0  0  0  0  0  0  0  0  0  0  0
+   -3.3849    0.7302    0.0000 L   0  0  0  0  0  0  0  0  0  0  0  0
+   -1.7451    1.4600    0.0000 L   0  0  0  0  0  0  0  0  0  0  0  0
+  1  2  2  0  0  0  0
+  2  3  1  0  0  0  0
+  3  4  2  0  0  0  0
+  4  5  1  0  0  0  0
+  5  6  2  0  0  0  0
+  6  1  1  0  0  0  0
+  6  7  1  0  0  0  0
+  8  9  2  0  0  0  0
+  2  8  1  0  0  0  0
+  9 10  1  0  0  0  0
+  3 10  1  0  0  0  0
+  9 11  1  0  0  0  0
+  8 12  1  0  0  0  0
+M  ALS   7 10 F H   C   N   O   F   P   S   Cl  Br  I   
+M  ALS  11 10 F H   C   N   O   F   P   S   Cl  Br  I   
+M  ALS  12 10 F H   C   N   O   F   P   S   Cl  Br  I   
+M  END
+)CTAB"_ctab;
+  auto mol = R"CTAB(
+  MJ201100                      
+
+ 13 14  0  0  0  0  0  0  0  0999 V2000
+   -0.6112    0.3665    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -1.3648    0.0310    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -1.4510   -0.7895    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -0.7836   -1.2744    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -0.0299   -0.9389    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+    0.0562   -0.1183    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+    0.8099    0.2172    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
+   -2.1184    0.3666    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -2.6705   -0.2464    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -2.2580   -0.9608    0.0000 N   0  0  0  0  0  0  0  0  0  0  0  0
+    0.6374   -1.4238    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
+    0.8961    1.0377    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+    0.5512   -2.2443    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+  1  2  2  0  0  0  0
+  2  3  1  0  0  0  0
+  3  4  2  0  0  0  0
+  4  5  1  0  0  0  0
+  5  6  2  0  0  0  0
+  6  1  1  0  0  0  0
+  8  9  2  0  0  0  0
+  2  8  1  0  0  0  0
+  9 10  1  0  0  0  0
+  3 10  1  0  0  0  0
+  6  7  1  0  0  0  0
+  5 11  1  0  0  0  0
+  7 12  1  0  0  0  0
+ 11 13  1  0  0  0  0
+M  END
+)CTAB"_ctab;
+  auto matchVect = RDDepict::generateDepictionMatching2DStructure(
+      *mol, *scaffold, -1, nullptr, false, false, true);
+  TEST_ASSERT(mol->getNumConformers() == 1);
+  TEST_ASSERT(matchVect.size() == 10);
   BOOST_LOG(rdInfoLog) << "Finished" << std::endl;
 }
 
@@ -1443,7 +1518,7 @@ M  END)RES"_ctab;
     TEST_ASSERT(RDKit::feq(bond4_11Conf2.y, bond4_11Conf3.y, 1.e-3));
   }
   {
-    std::string zeroCoordCTab = R"RES(
+    auto zeroCoordCTab = R"RES(
      RDKit          2D
 
   6  6  0  0  0  0  0  0  0  0999 V2000
@@ -1466,6 +1541,70 @@ M  END
     TEST_ASSERT(res < 0.);
     TEST_ASSERT(MolToMolBlock(*zeroCoordBenzene) == zeroCoordCTab);
   }
+  {
+    // cyclopentadiene which is already straight should not be biased
+    // towards a 30-degree angle rotate since it has no bonds
+    // whose angle with the X axis is multiple of 60 degrees
+    auto cpSittingOnHorizontalBondCTab = R"RES(
+  MJ201100                      
+
+  5  5  0  0  0  0  0  0  0  0999 V2000
+   -2.3660    0.3892    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -3.0334   -0.0957    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -2.7785   -0.8803    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -1.9535   -0.8803    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   -1.6986   -0.0957    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+  1  2  1  0  0  0  0
+  1  5  1  0  0  0  0
+  2  3  2  0  0  0  0
+  3  4  1  0  0  0  0
+  4  5  2  0  0  0  0
+M  END
+)RES";
+    std::unique_ptr<RWMol> cpSittingOnHorizontalBond(MolBlockToMol(cpSittingOnHorizontalBondCTab));
+    std::unique_ptr<RWMol> cpSittingOnHorizontalBondCopy(new RWMol(*cpSittingOnHorizontalBond));
+    RDDepict::straightenDepiction(*cpSittingOnHorizontalBond);
+    TEST_ASSERT(MolAlign::CalcRMS(*cpSittingOnHorizontalBond, *cpSittingOnHorizontalBondCopy) < 1.e-3);
+    RDGeom::Transform3D trans;
+    // rotate by 90 degrees
+    trans.SetRotation(0.5 * M_PI, RDGeom::Z_Axis);
+    MolTransforms::transformConformer(cpSittingOnHorizontalBond->getConformer(), trans);
+    cpSittingOnHorizontalBondCopy.reset(new RWMol(*cpSittingOnHorizontalBond));
+    RDDepict::straightenDepiction(*cpSittingOnHorizontalBond);
+    TEST_ASSERT(MolAlign::CalcRMS(*cpSittingOnHorizontalBond, *cpSittingOnHorizontalBondCopy) < 1.e-3);
+  }
+}
+
+void testValidRingSystemTemplates() {
+  BOOST_LOG(rdInfoLog)
+      << "-----------------------\n Test that ring system templates are valid "
+      << std::endl;
+  constexpr double RDKIT_BOND_LEN = 1.5;
+  for (auto &smiles : TEMPLATE_SMILES) {
+    ROMol *mol = SmilesToMol(smiles);
+    RDDepict::CoordinateTemplates::assertValidTemplate(*mol, smiles);
+
+    // also check whether the bonds in the template are the correct length
+    double avg_length = 0.0;
+    const Conformer &conf = mol->getConformer();
+    for (auto &bond : mol->bonds()) {
+      auto bond_length = (conf.getAtomPos(bond->getBeginAtomIdx()) -
+                          conf.getAtomPos(bond->getEndAtomIdx()))
+                             .length();
+      avg_length += bond_length;
+    }
+    avg_length /= mol->getNumBonds();
+
+    // this is a loose tolerance, since some complicated ring systems may have
+    // odd bond lengths
+    bool valid_length = RDKit::feq(avg_length, RDKIT_BOND_LEN, 0.1);
+    if (!valid_length) {
+      BOOST_LOG(rdWarningLog)
+          << "Template has invalid average bond "
+          << "length of " << avg_length << ": " << smiles << std::endl;
+    }
+    TEST_ASSERT(valid_length);
+  }
 }
 
 int main() {
@@ -1671,6 +1810,7 @@ int main() {
   testGenerate2DDepictionRefPatternMatchVect();
   testGenerate2DDepictionAllowRGroups();
   testNormalizeStraighten();
+  testValidRingSystemTemplates();
 
   return (0);
 }
