@@ -491,4 +491,237 @@ M  END)CTAB"_ctab;
       CHECK(cid2 >= 0);
     }
   }
+}
+
+TEST_CASE("double bond stereo not honored in conformer generator") {
+  SECTION("mol 1 basics") {
+    // this test used to fail
+    // from the platinum set
+    auto m = "O=C1OCC/C=C/CC/C=C/C(=N/OCC(=O)N2CCCCC2)Cc2cc(O)cc(O)c21"_smiles;
+    REQUIRE(m);
+    RWMol cp(*m);
+    MolOps::addHs(cp);
+    DGeomHelpers::EmbedParameters ps = DGeomHelpers::ETKDGv3;
+    ps.randomSeed = 0xf00d + 81;
+    auto cid = DGeomHelpers::EmbedMolecule(cp, ps);
+    REQUIRE(cid >= 0);
+    MolOps::assignStereochemistryFrom3D(cp);
+    // std::cerr << MolToMolBlock(cp) << std::endl;
+    for (const auto bnd : cp.bonds()) {
+      if (bnd->getBondType() == Bond::BondType::DOUBLE) {
+        INFO(bnd->getIdx());
+        CHECK(bnd->getStereo() ==
+              m->getBondWithIdx(bnd->getIdx())->getStereo());
+      }
+    }
+  }
+  SECTION("mol 1 multiple loops") {
+    // from the platinum set
+    auto m = "O=C1OCC/C=C/CC/C=C/C(=N/OCC(=O)N2CCCCC2)Cc2cc(O)cc(O)c21"_smiles;
+    REQUIRE(m);
+    RWMol cp(*m);
+    MolOps::addHs(cp);
+    DGeomHelpers::EmbedParameters ps = DGeomHelpers::ETKDGv3;
+    for (unsigned int iter = 0; iter < 10; ++iter) {
+      RWMol lcp(cp);
+      ps.randomSeed = 0xf00d + iter;
+      auto cid = DGeomHelpers::EmbedMolecule(lcp, ps);
+      REQUIRE(cid >= 0);
+      MolOps::assignStereochemistryFrom3D(lcp);
+      // std::cerr << MolToMolBlock(cp) << std::endl;
+      for (const auto bnd : lcp.bonds()) {
+        if (bnd->getBondType() == Bond::BondType::DOUBLE) {
+          INFO(iter);
+          CHECK(bnd->getStereo() ==
+                m->getBondWithIdx(bnd->getIdx())->getStereo());
+        }
+      }
+    }
+  }
+
+  SECTION("github #5913") {
+    auto m = "[H]/C(F)=C([H])\\C([H])=C(/[H])Br"_smiles;
+    REQUIRE(m);
+
+    RWMol cp(*m);
+    MolOps::addHs(cp);
+    DGeomHelpers::EmbedParameters ps = DGeomHelpers::ETKDGv3;
+    for (unsigned int iter = 0; iter < 50; ++iter) {
+      RWMol lcp(cp);
+      ps.randomSeed = 0 + iter;
+      auto cid = DGeomHelpers::EmbedMolecule(lcp, ps);
+      REQUIRE(cid >= 0);
+      MolOps::assignStereochemistryFrom3D(lcp);
+      // std::cerr << MolToMolBlock(cp) << std::endl;
+      for (const auto bnd : lcp.bonds()) {
+        if (bnd->getBondType() == Bond::BondType::DOUBLE) {
+          INFO(iter);
+          CHECK(bnd->getStereo() ==
+                m->getBondWithIdx(bnd->getIdx())->getStereo());
+        }
+      }
+    }
+  }
+
+  SECTION("github #5283") {
+    auto oVal = Chirality::getUseLegacyStereoPerception();
+    Chirality::setUseLegacyStereoPerception(false);
+    auto m =
+        "Cc3nn(CC(=O)N2CCN(c1ccccc1)CC2)c(C)c3/N=N\\c6ccc(CNC(=O)CCC(=O)Nc4cccc5C(=O)NCc45)cc6"_smiles;
+    REQUIRE(m);
+    RWMol cp(*m);
+    MolOps::addHs(cp);
+    DGeomHelpers::EmbedParameters ps = DGeomHelpers::ETKDGv3;
+    ps.enforceChirality = false;
+    for (unsigned int iter = 0; iter < 10; ++iter) {
+      INFO(iter);
+      RWMol lcp(cp);
+      ps.randomSeed = 140 + iter;
+      auto cid = DGeomHelpers::EmbedMolecule(lcp, ps);
+      REQUIRE(cid >= 0);
+      MolOps::assignStereochemistryFrom3D(lcp, cid, true);
+      auto bnd = lcp.getBondBetweenAtoms(22, 23);
+      REQUIRE(bnd);
+      REQUIRE(bnd->getBondType() == Bond::BondType::DOUBLE);
+      CHECK(bnd->getStereo() == m->getBondWithIdx(bnd->getIdx())->getStereo());
+    }
+    Chirality::setUseLegacyStereoPerception(oVal);
+  }
+}
+
+TEST_CASE("tracking failure causes"){SECTION("basics"){
+    auto mol =
+        "C=CC1=C(N)Oc2cc1c(-c1cc(C(C)O)cc(=O)cc1C1NCC(=O)N1)c(OC)c2OC"_smiles;
+REQUIRE(mol);
+MolOps::addHs(*mol);
+DGeomHelpers::EmbedParameters ps = DGeomHelpers::ETKDGv3;
+ps.randomSeed = 0xf00d;
+ps.trackFailures = true;
+ps.maxIterations = 50;
+ps.randomSeed = 42;
+auto cid = DGeomHelpers::EmbedMolecule(*mol, ps);
+CHECK(cid < 0);
+
+CHECK(ps.failures[DGeomHelpers::EmbedFailureCauses::INITIAL_COORDS] > 5);
+CHECK(ps.failures[DGeomHelpers::EmbedFailureCauses::ETK_MINIMIZATION] > 10);
+
+auto fail_cp = ps.failures;
+// make sure we reset the counts each time
+cid = DGeomHelpers::EmbedMolecule(*mol, ps);
+CHECK(ps.failures == fail_cp);
+}
+SECTION("chirality") {
+  auto mol = R"CTAB(
+  Ketcher  1102315302D 1   1.00000     0.00000     0
+
+ 10 11  0  0  1  0  0  0  0  0999 V2000
+   10.1340  -11.0250    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   10.1340  -12.0250    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   11.0000  -12.5250    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   11.8660  -12.0250    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   11.8660  -11.0250    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   11.0000  -10.5250    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
+   11.0000  -11.5250    0.0000 N   0  0  0  0  0  0  0  0  0  0  0  0
+   11.2588  -12.4909    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+    9.2680  -10.5250    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+   12.7629  -12.4673    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
+  1  6  1  0     0  0
+  1  2  1  0     0  0
+  2  3  1  0     0  0
+  3  4  1  0     0  0
+  4  5  1  0     0  0
+  5  6  1  0     0  0
+  1  7  1  0     0  0
+  7  8  1  0     0  0
+  8  4  1  0     0  0
+  1  9  1  1     0  0
+  4 10  1  1     0  0
+M  END
+)CTAB"_ctab;
+  REQUIRE(mol);
+  MolOps::addHs(*mol);
+  DGeomHelpers::EmbedParameters ps = DGeomHelpers::ETKDGv3;
+  ps.randomSeed = 0xf00d;
+  ps.trackFailures = true;
+  ps.maxIterations = 50;
+  auto cid = DGeomHelpers::EmbedMolecule(*mol, ps);
+  CHECK(cid < 0);
+  CHECK(ps.failures[DGeomHelpers::EmbedFailureCauses::INITIAL_COORDS] > 5);
+  CHECK(ps.failures[DGeomHelpers::EmbedFailureCauses::FINAL_CHIRAL_BOUNDS] > 5);
+}
+
+#ifdef RDK_TEST_MULTITHREADED
+SECTION("multithreaded") {
+  auto mol =
+      "C=CC1=C(N)Oc2cc1c(-c1cc(C(C)O)cc(=O)cc1C1NCC(=O)N1)c(OC)c2OC"_smiles;
+  REQUIRE(mol);
+  MolOps::addHs(*mol);
+  DGeomHelpers::EmbedParameters ps = DGeomHelpers::ETKDGv3;
+  ps.randomSeed = 0xf00d;
+  ps.trackFailures = true;
+  ps.maxIterations = 10;
+  ps.randomSeed = 42;
+  auto cids = DGeomHelpers::EmbedMultipleConfs(*mol, 20, ps);
+
+  DGeomHelpers::EmbedParameters ps2 = ps;
+  ps2.numThreads = 4;
+
+  auto cids2 = DGeomHelpers::EmbedMultipleConfs(*mol, 20, ps2);
+  CHECK(cids2 == cids);
+
+  CHECK(ps.failures == ps2.failures);
+}
+#endif
+}
+
+TEST_CASE("Github #5883: confgen failing for chiral N in a three ring") {
+  SECTION("basics1") {
+    auto mol = "N1[C@H-]C1"_smiles;
+    REQUIRE(mol);
+    MolOps::addHs(*mol);
+    mol->getAtomWithIdx(1)->setChiralTag(Atom::ChiralType::CHI_TETRAHEDRAL_CCW);
+    DGeomHelpers::EmbedParameters ps = DGeomHelpers::ETKDGv3;
+    ps.randomSeed = 42;
+    ps.maxIterations = 1;
+    auto cid = DGeomHelpers::EmbedMolecule(*mol, ps);
+    CHECK(cid >= 0);
+  }
+  SECTION("basics2") {
+    auto mol = "N1[N@H]C1"_smiles;
+    REQUIRE(mol);
+    MolOps::addHs(*mol);
+    mol->getAtomWithIdx(1)->setChiralTag(Atom::ChiralType::CHI_TETRAHEDRAL_CCW);
+    DGeomHelpers::EmbedParameters ps = DGeomHelpers::ETKDGv3;
+    ps.randomSeed = 42;
+    ps.maxIterations = 1;
+    auto cid = DGeomHelpers::EmbedMolecule(*mol, ps);
+    CHECK(cid >= 0);
+  }
+  SECTION("no ring") {
+    auto mol = "N[C@H-]C"_smiles;
+    REQUIRE(mol);
+    MolOps::addHs(*mol);
+    mol->getAtomWithIdx(1)->setChiralTag(Atom::ChiralType::CHI_TETRAHEDRAL_CCW);
+    DGeomHelpers::EmbedParameters ps = DGeomHelpers::ETKDGv3;
+    ps.randomSeed = 42;
+    ps.maxIterations = 1;
+    auto cid = DGeomHelpers::EmbedMolecule(*mol, ps);
+    CHECK(cid >= 0);
+  }
+}
+
+TEST_CASE("Github #6365: cannot generate conformers for PF6- or SF6") {
+  SECTION("basics") {
+    std::vector<std::string> smileses = {"S(F)(F)(F)(F)(F)F",
+                                         "[P-](F)(F)(F)(F)(F)F"};
+    for (const auto &smi : smileses) {
+      std::unique_ptr<RWMol> mol{SmilesToMol(smi)};
+      REQUIRE(mol);
+      DGeomHelpers::EmbedParameters ps = DGeomHelpers::ETKDGv3;
+      ps.randomSeed = 42;
+      ps.useRandomCoords = true;
+      auto cid = DGeomHelpers::EmbedMolecule(*mol, ps);
+      CHECK(cid >= 0);
+    }
+  }
 }
\ No newline at end of file
