@@ -17,6 +17,7 @@
 #include <RDGeneral/Invariant.h>
 #include <GraphMol/RDKitQueries.h>
 #include <GraphMol/SubstanceGroup.h>
+#include <GraphMol/Chirality.h>
 #include <RDGeneral/Ranking.h>
 #include <RDGeneral/LocaleSwitcher.h>
 
@@ -400,6 +401,7 @@ const std::string AtomGetMolFileSymbol(
             static_cast<ATOM_EQUALS_QUERY *>(atom->getQuery())->getVal() ==
                 1))) {
         res = "A";
+        queryListAtoms.set(atom->getIdx());
       } else if (atom->hasQuery() &&
                  (atom->getQuery()->getTypeLabel() == "Q" ||
                   (atom->getQuery()->getNegation() &&
@@ -530,9 +532,12 @@ bool hasNonDefaultValence(const Atom *atom) {
   if (atom->getNumRadicalElectrons() != 0) {
     return true;
   }
-  if (atom->hasQuery()) {
+  // for queries and atoms which don't have computed properties, the answer is
+  // always no:
+  if (atom->hasQuery() || atom->needsUpdatePropertyCache()) {
     return false;
   }
+
   if (atom->getAtomicNum() == 1 ||
       SmilesWrite ::inOrganicSubset(atom->getAtomicNum())) {
     // for the ones we "know", we may have to specify the valence if it's
@@ -795,21 +800,20 @@ void GetMolFileBondStereoInfo(const Bond *bond, const INT_MAP_INT &wedgeBonds,
     // as "any", this was sf.net issue 2963522.
     // two caveats to this:
     // 1) if it's a ring bond, we'll only put the "any"
-    //    in the mol block if the user specifically asked for it.
-    //    Constantly seeing crossed bonds in rings, though maybe
-    //    technically correct, is irritating.
+    //    in the mol block if the ring size is >=
+    //    Chirality::minRingSizeForDoubleBondStereo.
+    ///   Constantly seeing crossed
+    //    bonds in rings, though maybe technically correct, is irritating.
     // 2) if it's a terminal bond (where there's no chance of
     //    stereochemistry anyway), we also skip the any.
     //    this was sf.net issue 3009756
     if (bond->getStereo() <= Bond::STEREOANY) {
       if (bond->getStereo() == Bond::STEREOANY) {
         dirCode = 3;
-      } else if (!(bond->getOwningMol().getRingInfo()->numBondRings(
-                     bond->getIdx())) &&
-                 bond->getBeginAtom()->getDegree() > 1 &&
-                 bond->getEndAtom()->getDegree() > 1) {
+      } else if (Chirality::detail::isBondPotentialStereoBond(bond)) {
         // we don't know that it's explicitly unspecified (covered above with
         // the ==STEREOANY check)
+
         // look to see if one of the atoms has a bond with direction set
         if (bond->getBondDir() == Bond::EITHERDOUBLE) {
           dirCode = 3;
@@ -1057,8 +1061,28 @@ int BondStereoCodeV2000ToV3000(int dirCode) {
   }
 }
 
+namespace {
+void createSMARTSQSubstanceGroups(ROMol &mol) {
+  for (const auto atom : mol.atoms()) {
+    std::string sma;
+    if (atom->hasQuery() &&
+        atom->getPropIfPresent(common_properties::MRV_SMA, sma) &&
+        !sma.empty()) {
+      SubstanceGroup sg(&mol, "DAT");
+      sg.setProp("QUERYTYPE", "SMARTSQ");
+      sg.setProp("QUERYOP", "=");
+      std::vector<std::string> dataFields{sma};
+      sg.setProp("DATAFIELDS", dataFields);
+      sg.addAtomWithIdx(atom->getIdx());
+      addSubstanceGroup(mol, sg);
+    }
+  }
+}
+}  // namespace
+
 void moveAdditionalPropertiesToSGroups(RWMol &mol) {
   GenericGroups::convertGenericQueriesToSubstanceGroups(mol);
+  createSMARTSQSubstanceGroups(mol);
 }
 
 const std::string GetV3000MolFileBondLine(const Bond *bond,
@@ -1359,7 +1383,7 @@ std::string MolToMolBlock(const ROMol &mol, bool includeStereo, int confId,
   if (trwmol.needsUpdatePropertyCache()) {
     trwmol.updatePropertyCache(false);
   }
-  if (kekulize) {
+  if (kekulize && mol.getNumBonds()) {
     MolOps::Kekulize(trwmol);
   }
 
