@@ -259,8 +259,10 @@ void ConnectTheDots(RWMol *mol, unsigned int flags) {
 }
 
 // These are macros to allow their use in C++ constants
-constexpr int BCNAM(char A, char B, char C) { return (A << 16) | (B << 8) | C; }
-constexpr int BCATM(char A, char B, char C, char D) {
+constexpr unsigned BCNAM(char A, char B, char C) {
+  return (A << 16) | (B << 8) | C;
+}
+constexpr unsigned BCATM(char A, char B, char C, char D) {
   return (A << 24) | (B << 16) | (C << 8) | D;
 }
 
@@ -273,6 +275,7 @@ static bool StandardPDBDoubleBond(unsigned int rescode, unsigned int atm1,
   }
 
   switch (rescode) {
+    case BCNAM('A', 'C', 'E'):
     case BCNAM('A', 'L', 'A'):
     case BCNAM('C', 'Y', 'S'):
     case BCNAM('G', 'L', 'Y'):
