@@ -408,5 +408,28 @@ std::vector<std::tuple<Point2D, Point2D, Point2D, Point2D>> getWavyLineSegments(
   return res;
 }
 
+RDKIT_MOLDRAW2D_EXPORT void calcArrowHead(Point2D &arrowEnd, Point2D &arrow1,
+                                          Point2D &arrow2,
+                                          const Point2D &arrowBegin,
+                                          bool asPolygon, double frac,
+                                          double angle) {
+  auto delta = arrowBegin - arrowEnd;
+  double cos_angle = std::cos(angle), sin_angle = std::sin(angle);
+  // to have the arrowhead a consistent fraction of the line length, we need
+  // the hypotenuse
+  frac /= cos_angle;
+  if (asPolygon) {
+    // allow for the mitring, using an empirically derived guess.
+    arrowEnd += delta * 0.1;
+  }
+  arrow1 = arrowEnd;
+  arrow1.x += frac * (delta.x * cos_angle + delta.y * sin_angle);
+  arrow1.y += frac * (delta.y * cos_angle - delta.x * sin_angle);
+
+  arrow2 = arrowEnd;
+  arrow2.x += frac * (delta.x * cos_angle - delta.y * sin_angle);
+  arrow2.y += frac * (delta.y * cos_angle + delta.x * sin_angle);
+}
+
 }  // namespace MolDraw2D_detail
 }  // namespace RDKit
