@@ -1618,4 +1618,36 @@ TEST_CASE("starting from CXSMILES") {
       CHECK(MolToSmiles(*newmol) == "*CC1CC2CC(*)N(*)CN2CN1C*");
     }
   }
-}
\ No newline at end of file
+}
+
+TEST_CASE(
+    "Github #6014: MolEnumerator is not propagating query information to molecules ") {
+  SECTION("basics") {
+    auto m = "O-[CX4]-[CH3] |LN:1:1.5|"_smarts;
+    REQUIRE(m);
+    auto bundle = MolEnumerator::enumerate(*m);
+    CHECK(bundle.size() == 5);
+    auto q0 = bundle[0];
+    REQUIRE(q0);
+    CHECK(q0->getAtomWithIdx(0)->hasQuery());
+    CHECK(MolToSmarts(*q0) == "O-[C&X4]-[C&H3]");
+    CHECK(MolToSmarts(*bundle[1]) == "O-[C&X4]-[C&X4]-[C&H3]");
+  }
+}
+
+TEST_CASE(
+    "Github #6432: MolEnumerate should clear the reaction properties on its results") {
+  SECTION("basics") {
+    auto m = "COC |LN:1:1.3|"_smarts;
+    REQUIRE(m);
+    auto bundle = MolEnumerator::enumerate(*m);
+    CHECK(bundle.size() == 3);
+    auto q0 = bundle[0];
+    REQUIRE(q0);
+    for (const auto atom : q0->atoms()) {
+      CHECK(!atom->hasProp(common_properties::reactantAtomIdx));
+      CHECK(!atom->hasProp(common_properties::reactionMapNum));
+      CHECK(!atom->hasProp("was_dummy"));
+    }
+  }
+}
