@@ -6,11 +6,16 @@
 //  The contents are covered by the terms of the BSD license
 //  which is included in the file license.txt, found at the root
 //  of the RDKit source tree.
+#ifndef _CRT_SECURE_NO_WARNINGS
 #define _CRT_SECURE_NO_WARNINGS
+#endif
 
 #include <cstring>
 #include <cstdlib>
 #include <cstdio>
+#include <deque>
+
+// #define VERBOSE_HASH 1
 
 #include <string>
 #include <GraphMol/RDKitBase.h>
@@ -24,9 +29,9 @@ namespace {
 
 void addCXExtensions(RDKit::RWMol *mol, std::string &result,
                      unsigned additionalSkips = 0) {
+  unsigned int cxflagsToSkip = additionalSkips | RDKit::SmilesWrite::CX_COORDS;
   auto cxext = RDKit::SmilesWrite::getCXExtensions(
-      *mol, RDKit::SmilesWrite::CX_ALL ^ RDKit::SmilesWrite::CX_COORDS ^
-                additionalSkips);
+      *mol, RDKit::SmilesWrite::CX_ALL ^ cxflagsToSkip);
   if (!cxext.empty()) {
     result += " " + cxext;
   }
@@ -290,7 +295,8 @@ void NormalizeHCount(Atom *aptr) {
   aptr->setNumExplicitHs(hcount);
 }
 
-std::string AnonymousGraph(RWMol *mol, bool elem, bool useCXSmiles) {
+std::string AnonymousGraph(RWMol *mol, bool elem, bool useCXSmiles,
+                           unsigned cxFlagsToSkip = 0) {
   PRECONDITION(mol, "bad molecule");
   std::string result;
   int charge = 0;
@@ -322,13 +328,14 @@ std::string AnonymousGraph(RWMol *mol, bool elem, bool useCXSmiles) {
   result = MolToSmiles(*mol);
 
   if (useCXSmiles) {
-    addCXExtensions(mol, result, SmilesWrite::CX_RADICALS);
+    addCXExtensions(mol, result, cxFlagsToSkip | SmilesWrite::CX_RADICALS);
   }
 
   return result;
 }
 
-std::string MesomerHash(RWMol *mol, bool netq, bool useCXSmiles) {
+std::string MesomerHash(RWMol *mol, bool netq, bool useCXSmiles,
+                        unsigned cxFlagsToSkip = 0) {
   PRECONDITION(mol, "bad molecule");
   std::string result;
   char buffer[32];
@@ -358,12 +365,295 @@ std::string MesomerHash(RWMol *mol, bool netq, bool useCXSmiles) {
     result += buffer;
   }
   if (useCXSmiles) {
-    addCXExtensions(mol, result, SmilesWrite::CX_RADICALS);
+    addCXExtensions(mol, result, cxFlagsToSkip | SmilesWrite::CX_RADICALS);
   }
   return result;
 }
 
-std::string TautomerHash(RWMol *mol, bool proto, bool useCXSmiles) {
+namespace {
+// candidate atoms are either unsaturated or have implicit Hs
+bool isCandidateAtom(const Atom *aptr) {
+  return aptr->getTotalNumHs() || aptr->getIsAromatic() ||
+         queryAtomUnsaturated(aptr);
+}
+
+// atomic number > 1, not carbon
+bool isHeteroAtom(const Atom *aptr) {
+  auto atNum = aptr->getAtomicNum();
+  return atNum != 6 && atNum > 1;
+}
+
+// aromatic (flagged or bond order), double, or triple
+bool isUnsaturatedBond(const Bond *bptr) {
+  return bptr->getIsAromatic() ||
+         bptr->getBondType() == Bond::BondType::AROMATIC ||
+         bptr->getBondType() == Bond::BondType::DOUBLE ||
+         bptr->getBondType() == Bond::BondType::TRIPLE;
+}
+
+// potential tautomeric bonds are unsaturated and both atoms are candidates
+bool isPossibleTautomericBond(const Bond *bptr) {
+  return isUnsaturatedBond(bptr) && isCandidateAtom(bptr->getBeginAtom()) &&
+         isCandidateAtom(bptr->getEndAtom());
+}
+
+// a bond is a possible starting bond if it involves a candidate hetereoatom
+// (definition above) and an unsaturated atom
+bool isPossibleStartingBond(const Bond *bptr) {
+  auto heteroBeg = isHeteroAtom(bptr->getBeginAtom()) &&
+                   isCandidateAtom(bptr->getBeginAtom());
+  auto heteroEnd =
+      isHeteroAtom(bptr->getEndAtom()) && isCandidateAtom(bptr->getEndAtom());
+  // at least one atom has to be an eligible heteroatom:
+  if (!heteroBeg && !heteroEnd) {
+    return false;
+  }
+
+  auto unsatBeg = bptr->getBeginAtom()->getIsAromatic() ||
+                  queryAtomUnsaturated(bptr->getBeginAtom());
+  auto unsatEnd = bptr->getEndAtom()->getIsAromatic() ||
+                  queryAtomUnsaturated(bptr->getEndAtom());
+
+  // at least one has to be unsaturated:
+  if (!unsatBeg && !unsatEnd) {
+    return false;
+  }
+
+  return true;
+}
+
+// is one of the atom's bonds in the startBonds set?
+bool hasStartBond(const Atom *aptr, const boost::dynamic_bitset<> &startBonds) {
+  for (const auto nbr : aptr->getOwningMol().atomBonds(aptr)) {
+    if (startBonds[nbr->getIdx()]) {
+      return true;
+    }
+  }
+  return false;
+}
+
+// skip the neighbor bond if otherAtom isn't a candidate and doesn't have a
+// start bond OR if the bond is neither unsaturated nor conjugated and atom
+// doesn't have a start bond
+bool skipNeighborBond(const Atom *atom, const Atom *otherAtom,
+                      const Bond *nbrBond,
+                      const boost::dynamic_bitset<> &startBonds) {
+  return (
+      (!isCandidateAtom(otherAtom) && !hasStartBond(otherAtom, startBonds)) ||
+      (!isUnsaturatedBond(nbrBond) && !nbrBond->getIsConjugated() &&
+       !hasStartBond(atom, startBonds)));
+}
+}  // namespace
+
+std::string TautomerHashv2(RWMol *mol, bool proto, bool useCXSmiles,
+                           unsigned cxFlagsToSkip = 0) {
+  PRECONDITION(mol, "bad molecule");
+  std::string result;
+  unsigned int hcount = 0;
+  int charge = 0;
+
+  boost::dynamic_bitset<> bondsToModify(mol->getNumBonds());
+  boost::dynamic_bitset<> bondsConsidered(mol->getNumBonds());
+
+  boost::dynamic_bitset<> startBonds(mol->getNumBonds());
+  for (const auto bnd : mol->bonds()) {
+    startBonds.set(bnd->getIdx(), isPossibleStartingBond(bnd));
+  }
+#ifdef VERBOSE_HASH
+  std::cerr << " START BONDS: " << startBonds << std::endl;
+#endif
+  for (auto bptr : mol->bonds()) {
+    // If this has already been considered or is not a possible starting bond,
+    // then skip it
+    if (bondsToModify[bptr->getIdx()] || bondsConsidered[bptr->getIdx()] ||
+        !startBonds[bptr->getIdx()]) {
+      continue;
+    }
+    boost::dynamic_bitset<> conjSystem(mol->getNumBonds());
+    boost::dynamic_bitset<> atomsInSystem(mol->getNumAtoms());
+    unsigned int numDonorCs = 0;
+    unsigned int activeHeteroHs = 0;
+    std::deque<const Bond *> bq;
+    // also include eligible neighbor bonds:
+    for (const auto atm :
+         std::vector<const Atom *>{bptr->getBeginAtom(), bptr->getEndAtom()}) {
+      if (atm->getAtomicNum() == 6) {
+        if (atm->getTotalNumHs()) {
+          ++numDonorCs;
+        }
+      } else if (isHeteroAtom(atm)) {
+        activeHeteroHs += atm->getTotalNumHs();
+      }
+      for (const auto nbrBond : mol->atomBonds(atm)) {
+        if (nbrBond == bptr || bondsConsidered[nbrBond->getIdx()]) {
+          continue;
+        }
+        auto oatom = nbrBond->getOtherAtom(atm);
+
+        // if the bond is unsaturated or to an atom with free Hs, include it:
+        // if (isUnsaturatedBond(nbrBond) || oatom->getTotalNumHs()) {
+
+#ifdef VERBOSE_HASH
+        std::cerr << "  check neighbor1 " << nbrBond->getIdx() << " from "
+                  << atm->getIdx() << "-" << oatom->getIdx() << std::endl;
+        std::cerr << "    " << bondsConsidered[nbrBond->getIdx()] << " icao "
+                  << isCandidateAtom(oatom) << " hsbo "
+                  << hasStartBond(oatom, startBonds) << " unsat "
+                  << isUnsaturatedBond(nbrBond) << " icaa "
+                  << isCandidateAtom(atm) << " hsba "
+                  << hasStartBond(atm, startBonds) << std::endl;
+#endif
+
+        if (skipNeighborBond(atm, oatom, nbrBond, startBonds) &&
+            skipNeighborBond(oatom, atm, nbrBond, startBonds)) {
+          continue;
+        }
+
+#ifdef VERBOSE_HASH
+        std::cerr << "    push " << nbrBond->getIdx() << " "
+                  << nbrBond->getBeginAtomIdx() << "-"
+                  << nbrBond->getEndAtomIdx() << std::endl;
+        std::cerr << " #### SET1 " << bptr->getIdx() << std::endl;
+#endif
+        bq.push_back(nbrBond);
+
+        // now we know that we should consider this bond
+
+        bondsConsidered.set(bptr->getIdx());
+        conjSystem.set(bptr->getIdx());
+      }
+    }
+
+    while (!bq.empty()) {
+      auto bnd = bq.front();
+      bq.pop_front();
+      if (bondsConsidered[bnd->getIdx()]) {
+        continue;
+      }
+
+#ifdef VERBOSE_HASH
+      std::cerr << "BQ: " << bnd->getIdx() << ": " << bnd->getBeginAtomIdx()
+                << "-" << bnd->getEndAtomIdx() << std::endl;
+      std::cerr << " #### SET2 " << bnd->getIdx() << std::endl;
+#endif
+      bondsConsidered.set(bnd->getIdx());
+      conjSystem.set(bnd->getIdx());
+      for (const auto atm :
+           std::vector<const Atom *>{bnd->getBeginAtom(), bnd->getEndAtom()}) {
+        if (atomsInSystem[atm->getIdx()]) {
+          continue;
+        }
+        if (atm->getAtomicNum() == 6) {
+          if (atm->getTotalNumHs()) {
+            ++numDonorCs;
+            atomsInSystem.set(atm->getIdx());
+          }
+        } else if (atm->getAtomicNum() > 1) {
+          activeHeteroHs += atm->getTotalNumHs();
+          atomsInSystem.set(atm->getIdx());
+        }
+        for (auto nbrBnd : mol->atomBonds(atm)) {
+          if (nbrBnd == bnd) {
+            continue;
+          }
+          auto oatom = nbrBnd->getOtherAtom(atm);
+
+#ifdef VERBOSE_HASH
+          std::cerr << "  check neighbor " << nbrBnd->getIdx() << " from "
+                    << atm->getIdx() << "-" << oatom->getIdx() << std::endl;
+          std::cerr << "    " << bondsConsidered[nbrBnd->getIdx()] << " icao "
+                    << isCandidateAtom(oatom) << " hsbo "
+                    << hasStartBond(oatom, startBonds) << " unsat "
+                    << isUnsaturatedBond(nbrBnd) << " icaa "
+                    << isCandidateAtom(atm) << " hsba "
+                    << hasStartBond(atm, startBonds) << std::endl;
+#endif
+          if (bondsConsidered[nbrBnd->getIdx()] ||
+              (skipNeighborBond(atm, oatom, nbrBnd, startBonds) &&
+               skipNeighborBond(oatom, atm, nbrBnd, startBonds))) {
+            continue;
+          }
+          bq.push_back(nbrBnd);
+#ifdef VERBOSE_HASH
+          std::cerr << "  added!" << std::endl;
+#endif
+        }
+      }
+    }
+    // we need to have at least two bonds and include at least one active H
+    if (conjSystem.count() > 1 && (activeHeteroHs || numDonorCs)) {
+#ifdef VERBOSE_HASH
+      std::cerr << "CONJ: " << conjSystem << " hetero " << activeHeteroHs
+                << " donor " << possibleDonorCs << std::endl;
+#endif
+      bondsToModify |= conjSystem;
+    } else {
+#ifdef VERBOSE_HASH
+      std::cerr << "REJECT CONJ: " << conjSystem << " hetero " << activeHeteroHs
+                << " donor " << possibleDonorCs << std::endl;
+#endif
+    }
+  }
+#ifdef VERBOSE_HASH
+  std::cerr << "FINAL: " << bondsToModify << std::endl;
+#endif
+  boost::dynamic_bitset<> atomsToModify(mol->getNumAtoms());
+  if (bondsToModify.any()) {
+    for (auto bptr : mol->bonds()) {
+      if (!bondsToModify[bptr->getIdx()]) {
+        continue;
+      }
+      bptr->setIsAromatic(false);
+      bptr->setBondType(Bond::AROMATIC);
+      bptr->setStereo(Bond::BondStereo::STEREONONE);
+      atomsToModify.set(bptr->getBeginAtomIdx());
+      atomsToModify.set(bptr->getEndAtomIdx());
+    }
+  }
+
+  if (atomsToModify.any()) {
+    for (auto aptr : mol->atoms()) {
+      if (!atomsToModify[aptr->getIdx()]) {
+        continue;
+      }
+      charge += aptr->getFormalCharge();
+      hcount += aptr->getTotalNumHs();
+      aptr->setIsAromatic(false);
+      aptr->setFormalCharge(0);
+      aptr->setNoImplicit(true);
+      aptr->setNumExplicitHs(0);
+    }
+  }
+
+  if (!bondsToModify.empty() || !atomsToModify.empty()) {
+    MolOps::assignRadicals(*mol);
+    // we may have just destroyed some stereocenters/bonds
+    // clean that up:
+    bool cleanIt = true;
+    bool force = true;
+    MolOps::assignStereochemistry(*mol, cleanIt, force);
+  }
+
+  SmilesWriteParams ps;
+  ps.allBondsExplicit = true;
+  ps.allHsExplicit = true;
+  result = MolToSmiles(*mol, ps);
+  char buffer[32];
+  if (!proto) {
+    sprintf(buffer, "_%d_%d", hcount, charge);
+  } else {
+    sprintf(buffer, "_%d", hcount - charge);
+  }
+  result += buffer;
+  if (useCXSmiles) {
+    addCXExtensions(mol, result, cxFlagsToSkip | SmilesWrite::CX_RADICALS);
+  }
+
+  return result;
+}
+
+std::string TautomerHash(RWMol *mol, bool proto, bool useCXSmiles,
+                         unsigned cxFlagsToSkip = 0) {
   PRECONDITION(mol, "bad molecule");
   std::string result;
   char buffer[32];
@@ -405,7 +695,7 @@ std::string TautomerHash(RWMol *mol, bool proto, bool useCXSmiles) {
   }
   result += buffer;
   if (useCXSmiles) {
-    addCXExtensions(mol, result, SmilesWrite::CX_RADICALS);
+    addCXExtensions(mol, result, cxFlagsToSkip | SmilesWrite::CX_RADICALS);
   }
 
   return result;
@@ -473,9 +763,12 @@ bool HasNbrInScaffold(Atom *aptr, unsigned char *is_in_scaffold) {
   return false;
 }
 
-std::string ExtendedMurckoScaffold(RWMol *mol, bool useCXSmiles) {
+std::string ExtendedMurckoScaffold(RWMol *mol, bool useCXSmiles,
+                                   unsigned cxFlagsToSkip = 0) {
   PRECONDITION(mol, "bad molecule");
-  RDKit::MolOps::fastFindRings(*mol);
+  if (!mol->getRingInfo()->isInitialized()) {
+    MolOps::fastFindRings(*mol);
+  }
 
   unsigned int maxatomidx = mol->getNumAtoms();
   auto *is_in_scaffold = (unsigned char *)alloca(maxatomidx);
@@ -514,12 +807,13 @@ std::string ExtendedMurckoScaffold(RWMol *mol, bool useCXSmiles) {
   std::string result;
   result = MolToSmiles(*mol);
   if (useCXSmiles) {
-    addCXExtensions(mol, result, SmilesWrite::CX_RADICALS);
+    addCXExtensions(mol, result, cxFlagsToSkip | SmilesWrite::CX_RADICALS);
   }
   return result;
 }
 
-std::string MurckoScaffoldHash(RWMol *mol, bool useCXSmiles) {
+std::string MurckoScaffoldHash(RWMol *mol, bool useCXSmiles,
+                               unsigned cxFlagsToSkip = 0) {
   PRECONDITION(mol, "bad molecule");
   std::vector<Atom *> for_deletion;
   do {
@@ -557,7 +851,7 @@ std::string MurckoScaffoldHash(RWMol *mol, bool useCXSmiles) {
   std::string result;
   result = MolToSmiles(*mol);
   if (useCXSmiles) {
-    addCXExtensions(mol, result, SmilesWrite::CX_RADICALS);
+    addCXExtensions(mol, result, cxFlagsToSkip | SmilesWrite::CX_RADICALS);
   }
   return result;
 }
@@ -685,13 +979,16 @@ void ClearEZStereo(Atom *atm) {
   }
 }
 
-std::string RegioisomerHash(RWMol *mol, bool useCXSmiles) {
+std::string RegioisomerHash(RWMol *mol, bool useCXSmiles,
+                            unsigned cxFlagsToSkip = 0) {
   PRECONDITION(mol, "bad molecule");
 
   // we need a copy of the molecule so that we can loop over the bonds of
   // something while modifying something else
-  RDKit::MolOps::fastFindRings(*mol);
   RDKit::ROMol molcpy(*mol);
+  if (molcpy.getRingInfo()->isInitialized()) {
+    MolOps::fastFindRings(molcpy);
+  }
   for (int i = molcpy.getNumBonds() - 1; i >= 0; --i) {
     auto bptr = molcpy.getBondWithIdx(i);
     int split = RegioisomerBond(bptr);
@@ -734,7 +1031,7 @@ std::string RegioisomerHash(RWMol *mol, bool useCXSmiles) {
 
   std::string result = MolToSmiles(*mol);
   if (useCXSmiles) {
-    addCXExtensions(mol, result);
+    addCXExtensions(mol, result, cxFlagsToSkip);
   }
 
   return result;
@@ -859,7 +1156,8 @@ std::string ArthorSubOrderHash(RWMol *mol) {
 }
 }  // namespace
 
-std::string MolHash(RWMol *mol, HashFunction func, bool useCXSmiles) {
+std::string MolHash(RWMol *mol, HashFunction func, bool useCXSmiles,
+                    unsigned cxFlagsToSkip) {
   PRECONDITION(mol, "bad molecule");
   std::string result;
   char buffer[32];
@@ -868,34 +1166,37 @@ std::string MolHash(RWMol *mol, HashFunction func, bool useCXSmiles) {
   switch (func) {
     default:
     case HashFunction::AnonymousGraph:
-      result = AnonymousGraph(mol, false, useCXSmiles);
+      result = AnonymousGraph(mol, false, useCXSmiles, cxFlagsToSkip);
       break;
     case HashFunction::ElementGraph:
-      result = AnonymousGraph(mol, true, useCXSmiles);
+      result = AnonymousGraph(mol, true, useCXSmiles, cxFlagsToSkip);
       break;
     case HashFunction::CanonicalSmiles:
       result = MolToSmiles(*mol);
       if (useCXSmiles) {
-        addCXExtensions(mol, result);
+        addCXExtensions(mol, result, cxFlagsToSkip);
       }
       break;
     case HashFunction::MurckoScaffold:
-      result = MurckoScaffoldHash(mol, useCXSmiles);
+      result = MurckoScaffoldHash(mol, useCXSmiles, cxFlagsToSkip);
       break;
     case HashFunction::ExtendedMurcko:
-      result = ExtendedMurckoScaffold(mol, useCXSmiles);
+      result = ExtendedMurckoScaffold(mol, useCXSmiles, cxFlagsToSkip);
       break;
     case HashFunction::Mesomer:
-      result = MesomerHash(mol, true, useCXSmiles);
+      result = MesomerHash(mol, true, useCXSmiles, cxFlagsToSkip);
       break;
     case HashFunction::RedoxPair:
-      result = MesomerHash(mol, false, useCXSmiles);
+      result = MesomerHash(mol, false, useCXSmiles, cxFlagsToSkip);
       break;
     case HashFunction::HetAtomTautomer:
-      result = TautomerHash(mol, false, useCXSmiles);
+      result = TautomerHash(mol, false, useCXSmiles, cxFlagsToSkip);
+      break;
+    case HashFunction::HetAtomTautomerv2:
+      result = TautomerHashv2(mol, false, useCXSmiles, cxFlagsToSkip);
       break;
     case HashFunction::HetAtomProtomer:
-      result = TautomerHash(mol, true, useCXSmiles);
+      result = TautomerHash(mol, true, useCXSmiles, cxFlagsToSkip);
       break;
     case HashFunction::MolFormula:
       result = NMMolecularFormula(mol);
@@ -923,7 +1224,7 @@ std::string MolHash(RWMol *mol, HashFunction func, bool useCXSmiles) {
       result = ArthorSubOrderHash(mol);
       break;
     case HashFunction::Regioisomer:
-      result = RegioisomerHash(mol, useCXSmiles);
+      result = RegioisomerHash(mol, useCXSmiles, cxFlagsToSkip);
       break;
   }
   return result;
