@@ -445,8 +445,7 @@ typedef enum {
 } SanitizeFlags;
 
 //! \brief carries out a collection of tasks for cleaning up a molecule and
-/// ensuring
-//! that it makes "chemical sense"
+//! ensuring that it makes "chemical sense"
 /*!
    This functions calls the following in sequence
      -# MolOps::cleanUp()
@@ -459,6 +458,7 @@ typedef enum {
      -# MolOps::setHybridization()
      -# MolOps::cleanupChirality()
      -# MolOps::adjustHs()
+     -# mol.updatePropertyCache()
 
    \param mol : the RWMol to be cleaned
 
@@ -583,6 +583,23 @@ RDKIT_GRAPHMOL_EXPORT int setAromaticity(
 */
 RDKIT_GRAPHMOL_EXPORT void cleanUp(RWMol &mol);
 
+//! Designed to be called by the sanitizer to handle special cases for
+//! organometallic species before valence is perceived
+/*!
+
+    \b Note that this function is experimental and may either change in behavior
+   or be replaced with something else in future releases.
+
+    Currently this:
+     - replaces single bonds between "hypervalent" organic atoms and metals with
+       dative bonds (this is following an IUPAC recommendation:
+       https://iupac.qmul.ac.uk/tetrapyrrole/TP8.html)
+
+   \param mol    the molecule of interest
+
+*/
+RDKIT_GRAPHMOL_EXPORT void cleanUpOrganometallics(RWMol &mol);
+
 //! Called by the sanitizer to assign radical counts to atoms
 RDKIT_GRAPHMOL_EXPORT void assignRadicals(RWMol &mol);
 
@@ -608,17 +625,17 @@ RDKIT_GRAPHMOL_EXPORT void adjustHs(RWMol &mol);
 
    \param mol             the molecule of interest
 
-   \param markAtomsBonds  if this is set to true, \c isAromatic boolean settings
-   on both the Bonds and Atoms are turned to false following the Kekulization,
-   otherwise they are left alone in their original state.
+   \param markAtomsBonds  if this is set to true, \c isAromatic boolean
+   settings on both the Bonds and Atoms are turned to false following the
+   Kekulization, otherwise they are left alone in their original state.
 
    \param maxBackTracks   the maximum number of attempts at back-tracking. The
    algorithm uses a back-tracking procedure to revisit a previous setting of
    double bond if we hit a wall in the kekulization process
 
    <b>Notes:</b>
-     - this does not modify query bonds which have bond type queries (like those
-       which come from SMARTS) or rings containing them.
+     - this does not modify query bonds which have bond type queries (like
+   those which come from SMARTS) or rings containing them.
      - even if \c markAtomsBonds is \c false the \c BondType for all modified
        aromatic bonds will be changed from \c RDKit::Bond::AROMATIC to \c
        RDKit::Bond::SINGLE or RDKit::Bond::DOUBLE during Kekulization.
@@ -632,9 +649,9 @@ RDKIT_GRAPHMOL_EXPORT void Kekulize(RWMol &mol, bool markAtomsBonds = true,
 
    \param mol             the molecule of interest
 
-   \param markAtomsBonds  if this is set to true, \c isAromatic boolean settings
-   on both the Bonds and Atoms are turned to false following the Kekulization,
-   otherwise they are left alone in their original state.
+   \param markAtomsBonds  if this is set to true, \c isAromatic boolean
+   settings on both the Bonds and Atoms are turned to false following the
+   Kekulization, otherwise they are left alone in their original state.
 
    \param maxBackTracks   the maximum number of attempts at back-tracking. The
    algorithm uses a back-tracking procedure to revisit a previous setting of
@@ -672,6 +689,8 @@ RDKIT_GRAPHMOL_EXPORT void setHybridization(ROMol &mol);
   \param res used to return the vector of rings. Each entry is a vector with
       atom indices.  This information is also stored in the molecule's
       RingInfo structure, so this argument is optional (see overload)
+  \param includeDativeBonds - determines whether or not dative bonds are used in
+      the ring finding.
 
   \return number of smallest rings found
 
@@ -707,10 +726,12 @@ RDKIT_GRAPHMOL_EXPORT void setHybridization(ROMol &mol);
   done. The extra rings this process adds can be quite useful.
 */
 RDKIT_GRAPHMOL_EXPORT int findSSSR(const ROMol &mol,
-                                   std::vector<std::vector<int>> &res);
+                                   std::vector<std::vector<int>> &res,
+                                   bool includeDativeBonds = false);
 //! \overload
-RDKIT_GRAPHMOL_EXPORT int findSSSR(
-    const ROMol &mol, std::vector<std::vector<int>> *res = nullptr);
+RDKIT_GRAPHMOL_EXPORT int findSSSR(const ROMol &mol,
+                                   std::vector<std::vector<int>> *res = nullptr,
+                                   bool includeDativeBonds = false);
 
 //! use a DFS algorithm to identify ring bonds and atoms in a molecule
 /*!
@@ -740,6 +761,8 @@ RDKIT_GRAPHMOL_EXPORT void findRingFamilies(const ROMol &mol);
   \param res used to return the vector of rings. Each entry is a vector with
       atom indices.  This information is also stored in the molecule's
       RingInfo structure, so this argument is optional (see overload)
+  \param includeDativeBonds - determines whether or not dative bonds are used in
+      the ring finding.
 
   \return the total number of rings = (new rings + old SSSRs)
 
@@ -748,9 +771,11 @@ RDKIT_GRAPHMOL_EXPORT void findRingFamilies(const ROMol &mol);
   first
 */
 RDKIT_GRAPHMOL_EXPORT int symmetrizeSSSR(ROMol &mol,
-                                         std::vector<std::vector<int>> &res);
+                                         std::vector<std::vector<int>> &res,
+                                         bool includeDativeBonds = false);
 //! \overload
-RDKIT_GRAPHMOL_EXPORT int symmetrizeSSSR(ROMol &mol);
+RDKIT_GRAPHMOL_EXPORT int symmetrizeSSSR(ROMol &mol,
+                                         bool includeDativeBonds = false);
 
 //! @}
 
@@ -895,9 +920,9 @@ RDKIT_GRAPHMOL_EXPORT void cleanupChirality(RWMol &mol);
 
 
   NOTE that this does not check to see if atoms are chiral centers (i.e. all
-  substituents are different), it merely sets the chiral type flags based on the
-  coordinates and atom ordering. Use \c assignStereochemistryFrom3D() if you
-  want chiral flags only on actual stereocenters.
+  substituents are different), it merely sets the chiral type flags based on
+  the coordinates and atom ordering. Use \c assignStereochemistryFrom3D() if
+  you want chiral flags only on actual stereocenters.
 */
 RDKIT_GRAPHMOL_EXPORT void assignChiralTypesFrom3D(
     ROMol &mol, int confId = -1, bool replaceExistingTags = true);
@@ -992,7 +1017,8 @@ RDKIT_GRAPHMOL_EXPORT void removeStereochemistry(ROMol &mol);
 
   This function is useful in the following situations:
     - when parsing a mol file; for the bonds marked here, coordinate
-      information on the neighbors can be used to indentify cis or trans states
+      information on the neighbors can be used to indentify cis or trans
+  states
     - when writing a mol file; bonds that can be cis/trans but not marked as
       either need to be specially marked in the mol file
     - finding double bonds with unspecified stereochemistry so they
@@ -1003,8 +1029,8 @@ RDKIT_GRAPHMOL_EXPORT void removeStereochemistry(ROMol &mol);
 */
 RDKIT_GRAPHMOL_EXPORT void findPotentialStereoBonds(ROMol &mol,
                                                     bool cleanIt = false);
-//! \brief Uses the molParity atom property to assign ChiralType to a molecule's
-//! atoms
+//! \brief Uses the molParity atom property to assign ChiralType to a
+//! molecule's atoms
 /*!
   \param mol                  the molecule of interest
   \param replaceExistingTags  if this flag is true, any existing atomic chiral
@@ -1022,12 +1048,50 @@ RDKIT_GRAPHMOL_EXPORT unsigned getNumAtomsWithDistinctProperty(
 //! returns whether or not a molecule needs to have Hs added to it.
 RDKIT_GRAPHMOL_EXPORT bool needsHs(const ROMol &mol);
 
+//! \brief Replaces haptic bond with explicit dative bonds.
+/*!
+ *
+ * @param mol the molecule of interest
+ *
+ * One way of showing haptic bonds (such as cyclopentadiene to iron in
+ * ferrocene) is to use a dummy atom with a dative bond to the iron atom with
+ * the bond labelled with the atoms involved in the organic end of the bond.
+ * Another way is to have explicit dative bonds from the atoms of the haptic
+ * group to the metal atom.  This function converts the former representation to
+ * the latter.
+ */
+RDKIT_GRAPHMOL_EXPORT ROMol *hapticBondsToDative(const ROMol &mol);
+
+//! \overload modifies molecule in place.
+RDKIT_GRAPHMOL_EXPORT void hapticBondsToDative(RWMol &mol);
+
+//! \brief Replaces explicit dative bonds with haptic.
+/*!
+ *
+ * @param mol the molecule of interest
+ *
+ * Does the reverse of hapticBondsToDative.  If there are multiple contiguous
+ * atoms attached by dative bonds to an atom (probably a metal atom), the dative
+ * bonds will be replaced by a dummy atom in their centre attached to the
+ * (metal) atom by a dative bond, which is labelled with ENDPTS of the atoms
+ * that had the original dative bonds.
+ */
+RDKIT_GRAPHMOL_EXPORT ROMol *dativeBondsToHaptic(const ROMol &mol);
+
+//! \overload modifies molecule in place.
+RDKIT_GRAPHMOL_EXPORT void dativeBondsToHaptic(RWMol &mol);
+
 namespace details {
 //! not recommended for use in other code
 RDKIT_GRAPHMOL_EXPORT void KekulizeFragment(
     RWMol &mol, const boost::dynamic_bitset<> &atomsToUse,
     boost::dynamic_bitset<> bondsToUse, bool markAtomsBonds = true,
     unsigned int maxBackTracks = 100);
+
+// If the bond is dative, and it has a common_properties::MolFileBondEndPts
+// prop, returns a vector of the indices of the atoms mentioned in the prop.
+RDKIT_GRAPHMOL_EXPORT std::vector<int> hapticBondEndpoints(const Bond *bond);
+
 }  // namespace details
 
 }  // namespace MolOps
