@@ -22,9 +22,7 @@ RDKIT_RGROUPDECOMPOSITION_EXPORT extern const std::string RLABEL_CORE_INDEX;
 RDKIT_RGROUPDECOMPOSITION_EXPORT extern const std::string SIDECHAIN_RLABELS;
 RDKIT_RGROUPDECOMPOSITION_EXPORT extern const std::string done;
 RDKIT_RGROUPDECOMPOSITION_EXPORT extern const std::string
-    UNLABELLED_CORE_ATTACHMENT;
-RDKIT_RGROUPDECOMPOSITION_EXPORT extern const std::string
-    MISSING_RGROUP;
+    UNLABELED_CORE_ATTACHMENT;
 
 const unsigned int EMPTY_CORE_LABEL = -100000;
 
