@@ -18,6 +18,7 @@
 #include <GraphMol/MolPickler.h>
 #include <GraphMol/QueryAtom.h>
 #include <GraphMol/QueryBond.h>
+#include <GraphMol/QueryOps.h>
 #include <GraphMol/Chirality.h>
 #include <GraphMol/SmilesParse/SmilesParse.h>
 #include <GraphMol/SmilesParse/SmilesWrite.h>
@@ -264,7 +265,7 @@ TEST_CASE("github #2257: writing cxsmiles", "[smiles][cxsmiles]") {
     auto mol = "C[C@H](F)[C@H](C)[C@@H](C)Br |a:1,o1:3,5|"_smiles;
     REQUIRE(mol);
     auto smi = MolToCXSmiles(*mol);
-    CHECK(smi == "C[C@@H]([C@H](C)F)[C@@H](C)Br |a:2,o1:1,5|");
+    CHECK(smi == "C[C@H](F)[C@@H](C)[C@H](C)Br |a:1,o1:3,5|");
   }
 
   SECTION("enhanced stereo 2") {
@@ -281,7 +282,7 @@ TEST_CASE("github #2257: writing cxsmiles", "[smiles][cxsmiles]") {
     auto smi = MolToCXSmiles(*mol);
     CHECK(
         smi ==
-        "C[C@@H]1[C@H](C)[C@H](C)N[C@H](C)[C@@H]1C1[C@@H](C)O[C@@H](C)[C@@H](C)[C@H]1C |a:9,o1:2,4,o2:14,16,&1:1,7,&2:11,18|");
+        "C[C@H]1N[C@H](C)[C@H](C2[C@H](C)O[C@H](C)[C@H](C)[C@@H]2C)[C@H](C)[C@H]1C |a:5,o1:1,18,o2:10,12,&1:3,16,&2:7,14|");
   }
 
   SECTION("enhanced stereo 4") {
@@ -695,9 +696,10 @@ TEST_CASE(
     auto smarts = MolToSmarts(*m);
     // this will change if/when the definition of the query changes, just have
     // to update then
-    CHECK(smarts ==
-          "[#6]-[!#2&!#5&!#6&!#7&!#8&!#9&!#10&!#14&!#15&!#16&!#17&!#18&!#33&!#"
-          "34&!#35&!#36&!#52&!#53&!#54&!#85&!#86&!#1]");
+    CHECK(
+        smarts ==
+        "[#6]-[!#0&!#2&!#5&!#6&!#7&!#8&!#9&!#10&!#14&!#15&!#16&!#17&!#18&!#33&!#"
+        "34&!#35&!#36&!#52&!#53&!#54&!#85&!#86&!#1]");
   }
   SECTION("serialization") {
     std::string pkl;
@@ -1728,7 +1730,7 @@ M  END)CTAB"_ctab;
 TEST_CASE("Github #4582 continued: double bonds and ring closures") {
   SECTION("basics") {
     auto mol = R"CTAB(
-  Mrv2108 10072106112D          
+  Mrv2108 10072106112D
 
  11 11  0  0  0  0            999 V2000
     1.2097   -1.0517    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
@@ -1958,7 +1960,7 @@ TEST_CASE("empty atom label block", "[cxsmiles]") {
   SECTION("basics") {
     auto m = R"CTAB(
   MJ201100
-                        
+
   8  8  0  0  0  0  0  0  0  0999 V2000
    -1.0491    1.5839    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
    -1.7635    1.1714    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
@@ -2113,6 +2115,27 @@ TEST_CASE("wiggly and wedged bonds in CXSMILES") {
     CHECK(bondcfg == 3);
     CHECK(m->getAtomWithIdx(1)->getChiralTag() ==
           Atom::ChiralType::CHI_TETRAHEDRAL_CCW);
+    invertMolBlockWedgingInfo(*m);
+    CHECK(m->getBondWithIdx(0)->getPropIfPresent("_MolFileBondCfg", bondcfg));
+    CHECK(bondcfg == 1);
+    reapplyMolBlockWedging(*m);
+    MolOps::assignChiralTypesFromBondDirs(*m);
+    CHECK(m->getAtomWithIdx(1)->getChiralTag() ==
+          Atom::ChiralType::CHI_TETRAHEDRAL_CW);
+    invertMolBlockWedgingInfo(*m);
+    CHECK(m->getBondWithIdx(0)->getPropIfPresent("_MolFileBondCfg", bondcfg));
+    CHECK(bondcfg == 3);
+    reapplyMolBlockWedging(*m);
+    MolOps::assignChiralTypesFromBondDirs(*m);
+    CHECK(m->getAtomWithIdx(1)->getChiralTag() ==
+          Atom::ChiralType::CHI_TETRAHEDRAL_CCW);
+    clearMolBlockWedgingInfo(*m);
+    m->getAtomWithIdx(1)->setChiralTag(Atom::ChiralType::CHI_UNSPECIFIED);
+    CHECK(!m->getBondWithIdx(0)->getPropIfPresent("_MolFileBondCfg", bondcfg));
+    reapplyMolBlockWedging(*m);
+    MolOps::assignChiralTypesFromBondDirs(*m);
+    CHECK(m->getAtomWithIdx(1)->getChiralTag() ==
+          Atom::ChiralType::CHI_UNSPECIFIED);
   }
 
   SECTION("writing examples") {
@@ -2134,7 +2157,7 @@ TEST_CASE("wiggly and wedged bonds in CXSMILES") {
 
   SECTION("writing wedges and dashes") {
     auto m = R"CTAB(
-  RDKit             2D          
+  RDKit             2D
 
   0  0  0     0  0            999 V3000
 M  V30 BEGIN CTAB
@@ -2179,7 +2202,7 @@ M  END
 
   SECTION("double bond stereo") {
     auto m_from_ctab = R"CTAB(
-  Mrv2211 09152216122D          
+  Mrv2211 09152216122D
 
   0  0  0     0  0            999 V3000
 M  V30 BEGIN CTAB
@@ -2217,6 +2240,8 @@ M  END
 }
 
 TEST_CASE("ring bond stereochemistry in CXSMILES") {
+  auto oval = Chirality::getUseLegacyStereoPerception();
+  Chirality::setUseLegacyStereoPerception(false);
   SECTION("basic reading") {
     std::vector<std::pair<std::string, Bond::BondStereo>> tests = {
         {"C1CCCC/C=C/CCC1 |t:5|", Bond::BondStereo::STEREOTRANS},
@@ -2227,9 +2252,7 @@ TEST_CASE("ring bond stereochemistry in CXSMILES") {
         {"C1CCCC/C=C/CCC1 |ctu:5|", Bond::BondStereo::STEREOANY},
     };
     for (const auto &[smi, val] : tests) {
-      SmilesParserParams ps;
-      ps.useLegacyStereo = false;
-      std::unique_ptr<RWMol> m{SmilesToMol(smi, ps)};
+      std::unique_ptr<RWMol> m{SmilesToMol(smi)};
       INFO(smi);
       REQUIRE(m);
       CHECK(m->getBondWithIdx(5)->getBondType() == Bond::BondType::DOUBLE);
@@ -2237,9 +2260,7 @@ TEST_CASE("ring bond stereochemistry in CXSMILES") {
     }
   }
   SECTION("read multiple values") {
-    SmilesParserParams ps;
-    ps.useLegacyStereo = false;
-    std::unique_ptr<RWMol> m{SmilesToMol("C1CCCCC=CCCC=1 |t:5,9|", ps)};
+    std::unique_ptr<RWMol> m{SmilesToMol("C1CCCCC=CCCC=1 |t:5,9|")};
     REQUIRE(m);
     CHECK(m->getBondWithIdx(5)->getBondType() == Bond::BondType::DOUBLE);
     CHECK(m->getBondWithIdx(5)->getStereo() == Bond::BondStereo::STEREOTRANS);
@@ -2251,9 +2272,7 @@ TEST_CASE("ring bond stereochemistry in CXSMILES") {
         {"C1=CCCCC1 |ctu:0|", Bond::BondStereo::STEREONONE},
         {"C1=CCCCC1 |c:0|", Bond::BondStereo::STEREONONE}};
     for (const auto &[smi, val] : tests) {
-      SmilesParserParams ps;
-      ps.useLegacyStereo = false;
-      std::unique_ptr<RWMol> m{SmilesToMol(smi, ps)};
+      std::unique_ptr<RWMol> m{SmilesToMol(smi)};
       INFO(smi);
       REQUIRE(m);
       CHECK(m->getBondWithIdx(0)->getBondType() == Bond::BondType::DOUBLE);
@@ -2271,9 +2290,7 @@ TEST_CASE("ring bond stereochemistry in CXSMILES") {
          "C=CCCCCCCCC"}  // we don't write the markers for non-ring bonds
     };
     for (const auto &[smi, val] : tests) {
-      SmilesParserParams ps;
-      ps.useLegacyStereo = false;
-      std::unique_ptr<RWMol> m{SmilesToMol(smi, ps)};
+      std::unique_ptr<RWMol> m{SmilesToMol(smi)};
       INFO(smi);
       REQUIRE(m);
       SmilesWriteParams ops;
@@ -2282,6 +2299,7 @@ TEST_CASE("ring bond stereochemistry in CXSMILES") {
       CHECK(cxsmi == val);
     }
   }
+  Chirality::setUseLegacyStereoPerception(oval);
 }
 
 TEST_CASE(
@@ -2339,4 +2357,67 @@ TEST_CASE("Github #5683: SMARTS bond ordering should be the same as SMILES") {
     auto sma = "O=C(C=CCC1CCCCC1)N1N=Cc2ccccc2C1c1ccccc1 |w:3.1|";
     CHECK_THROWS_AS(SmartsToMol(sma), SmilesParseException);
   }
+}
+
+TEST_CASE("SMARTS for molecule with zero order bond should match itself") {
+  auto m = "CN(<-[Li+])C"_smiles;
+  m->getBondBetweenAtoms(1, 2)->setBondType(Bond::BondType::ZERO);
+  const auto sma = MolToSmarts(*m);
+  std::unique_ptr<ROMol> q{SmartsToMol(sma)};
+  SubstructMatchParameters ps;
+  ps.maxMatches = 1;
+  CHECK(SubstructMatch(*m, *q, ps).size() == 1);
+}
+
+TEST_CASE("smilesSymbol in SMARTS", "[smarts][smilesSymbol]") {
+  // Probably just the first case is going to be useful to people
+  SECTION("smilesSymbol without queries") {
+    auto m = "CC*"_smiles;
+    REQUIRE(m);
+    m->getAtomWithIdx(2)->setProp(common_properties::smilesSymbol, "Xa");
+    CHECK(MolToSmarts(*m) == "[#6]-[#6]-[Xa]");
+  }
+  SECTION("smilesSymbol with query on other atoms") {
+    auto m = "[#6]cc"_smarts;
+    REQUIRE(m);
+    m->getAtomWithIdx(0)->setProp(common_properties::smilesSymbol, "Xa");
+    CHECK(MolToSmarts(*m) == "[Xa]cc");
+  }
+  SECTION("smilesSymbol with aromaticity query") {
+    auto m = "ccc"_smarts;
+    REQUIRE(m);
+    m->getAtomWithIdx(0)->setProp(common_properties::smilesSymbol, "Xa");
+    CHECK(MolToSmarts(*m) == "[Xa;a]cc");
+  }
+  SECTION("smilesSymbol with aliphatic query") {
+    auto m = "CCC"_smarts;
+    REQUIRE(m);
+    m->getAtomWithIdx(0)->setProp(common_properties::smilesSymbol, "Xa");
+    CHECK(MolToSmarts(*m) == "[Xa;A]CC");
+  }
+  SECTION("smilesSymbol with additional queries") {
+    auto m = "[#6]C[#6]"_smarts;
+    REQUIRE(m);
+    auto atom = m->getAtomWithIdx(1);
+
+    atom->expandQuery(makeAtomExplicitDegreeQuery(3), Queries::COMPOSITE_AND);
+    atom->expandQuery(makeAtomTotalValenceQuery(4), Queries::COMPOSITE_AND);
+    atom->expandQuery(makeAtomFormalChargeQuery(2), Queries::COMPOSITE_AND);
+
+    atom->setProp(common_properties::smilesSymbol, "Xa");
+
+    CHECK(MolToSmarts(*m) == "[#6][Xa;A&D3&v4&+2][#6]");
+  }
+  SECTION("degree query") {
+    auto m = "[X3]-C"_smarts;
+    REQUIRE(m);
+    m->getAtomWithIdx(0)->setProp(common_properties::smilesSymbol, "Xa");
+    CHECK(MolToSmarts(*m) == "[Xa;X3]-C");
+  }
+  SECTION("atom list") {
+    auto m = "[C,N,O]C"_smarts;
+    REQUIRE(m);
+    m->getAtomWithIdx(0)->setProp(common_properties::smilesSymbol, "Xa");
+    CHECK(MolToSmarts(*m) == "[Xa;C,N,O]C");
+  }
 }
\ No newline at end of file
