@@ -13,12 +13,20 @@
 
 // std bits
 #include <vector>
+
+#include <unordered_set>
 #include <functional>
 #include <unordered_map>
 #include <cstdint>
-#include "GraphMol/StereoGroup.h"
 #include <string>
 
+#include <boost/dynamic_bitset.hpp>
+#if BOOST_VERSION >= 107100
+#define RDK_INTERNAL_BITSET_HAS_HASH
+#endif
+
+#include <GraphMol/StereoGroup.h>
+
 namespace RDKit {
 class ROMol;
 class Atom;
@@ -59,6 +67,8 @@ struct RDKIT_SUBSTRUCTMATCH_EXPORT SubstructMatchParameters {
 
 RDKIT_SUBSTRUCTMATCH_EXPORT void updateSubstructMatchParamsFromJSON(
     SubstructMatchParameters &params, const std::string &json);
+RDKIT_SUBSTRUCTMATCH_EXPORT std::string substructMatchParamsToJSON(
+    const SubstructMatchParameters &params);
 
 //! Find a substructure match for a query in a molecule
 /*!
@@ -218,13 +228,21 @@ class RDKIT_SUBSTRUCTMATCH_EXPORT MolMatchFinalCheckFunctor {
   MolMatchFinalCheckFunctor(const ROMol &query, const ROMol &mol,
                             const SubstructMatchParameters &ps);
 
-  bool operator()(const std::uint32_t q_c[], const std::uint32_t m_c[]) const;
+  bool operator()(const std::uint32_t q_c[], const std::uint32_t m_c[]);
 
  private:
   const ROMol &d_query;
   const ROMol &d_mol;
   const SubstructMatchParameters &d_params;
   std::unordered_map<unsigned int, StereoGroup const *> d_molStereoGroups;
+#ifdef RDK_INTERNAL_BITSET_HAS_HASH
+  // Boost 1.71 added support for std::hash with dynamic_bitset.
+  using HashedStorageType = boost::dynamic_bitset<>;
+#else
+  // otherwise we use a less elegant solution
+  using HashedStorageType = std::string;
+#endif
+  std::unordered_set<HashedStorageType> matchesSeen;
 };
 
 }  // namespace RDKit
