@@ -42,13 +42,12 @@ class ScoreMatchesByDegreeOfCoreSubstitution {
         d_minIdx(-1),
         d_isSorted(false) {
     PRECONDITION(!matches.empty(), "matches must not be empty");
-    for (unsigned int i = 0; i < d_mol.getNumAtoms(); ++i) {
-      d_sumIndices += static_cast<double>(i);
-    }
+    auto na = d_mol.getNumAtoms();
+    d_sumIndices = static_cast<double>(na * (na + 1) / 2);
     unsigned int i = 0;
     d_matchIdxVsScore.reserve(d_matches.size());
     for (const auto &match : d_matches) {
-      d_matchIdxVsScore.emplace_back(std::make_pair(i++, computeScore(match)));
+      d_matchIdxVsScore.emplace_back(i++, computeScore(match));
     }
   }
   const RDKit::MatchVectType &getMostSubstitutedCoreMatch() {
@@ -79,8 +78,8 @@ class ScoreMatchesByDegreeOfCoreSubstitution {
   bool doesRGroupMatchHydrogen(const std::pair<int, int> &pair) const {
     const auto queryAtom = d_query.getAtomWithIdx(pair.first);
     const auto molAtom = d_mol.getAtomWithIdx(pair.second);
-    return (queryAtom->getAtomicNum() == 0 && queryAtom->getDegree() == 1 &&
-            molAtom->getAtomicNum() == 1);
+    return (molAtom->getAtomicNum() == 1 &&
+            isAtomTerminalRGroupOrQueryHydrogen(queryAtom));
   }
   double computeScore(const RDKit::MatchVectType &match) const {
     double penalty = 0.0;
@@ -217,7 +216,16 @@ std::vector<MatchVectType> sortMatchesByDegreeOfCoreSubstitution(
   return matchScorer.sortMatchesByDegreeOfCoreSubstitution();
 }
 
+bool isAtomTerminalRGroupOrQueryHydrogen(const Atom *atom) {
+  return atom->getDegree() == 1 &&
+         (atom->getAtomicNum() == 0 ||
+          (atom->hasQuery() &&
+           describeQuery(atom).find("AtomAtomicNum 1 = val") !=
+               std::string::npos));
+}
+
 #define PT_OPT_GET(opt) params.opt = pt.get(#opt, params.opt)
+#define PT_OPT_PUT(opt) pt.put(#opt, params.opt);
 
 void updateSubstructMatchParamsFromJSON(SubstructMatchParameters &params,
                                         const std::string &json) {
@@ -238,4 +246,21 @@ void updateSubstructMatchParamsFromJSON(SubstructMatchParameters &params,
   PT_OPT_GET(numThreads);
 }
 
+std::string substructMatchParamsToJSON(const SubstructMatchParameters &params) {
+  boost::property_tree::ptree pt;
+
+  PT_OPT_PUT(useChirality);
+  PT_OPT_PUT(useEnhancedStereo);
+  PT_OPT_PUT(aromaticMatchesConjugated);
+  PT_OPT_PUT(useQueryQueryMatches);
+  PT_OPT_PUT(recursionPossible);
+  PT_OPT_PUT(uniquify);
+  PT_OPT_PUT(maxMatches);
+  PT_OPT_PUT(numThreads);
+
+  std::stringstream ss;
+  boost::property_tree::json_parser::write_json(ss, pt);
+  return ss.str();
+}
+
 }  // namespace RDKit
