@@ -553,6 +553,16 @@ void initCanonAtoms(const ROMol &mol, std::vector<Canon::canon_atom> &atoms,
     atoms[i].bonds.reserve(atoms[i].degree);
     getBonds(mol, atoms[i].atom, atoms[i].bonds, includeChirality, atoms);
   }
+  if (includeChirality) {
+    unsigned int sgidx = 1;
+    for (auto &sg : mol.getStereoGroups()) {
+      for (auto atom : sg.getAtoms()) {
+        atoms[atom->getIdx()].whichStereoGroup = sgidx;
+        atoms[atom->getIdx()].typeOfStereoGroup = sg.getGroupType();
+      }
+      ++sgidx;
+    }
+  }
 }
 namespace detail {
 
