@@ -121,17 +121,17 @@ std::string JSMol::get_inchi() const {
   ExtraInchiReturnValues rv;
   return MolToInchi(*d_mol, rv);
 }
-std::string JSMol::get_molblock() const {
+std::string JSMol::get_molblock(const std::string &details) const {
   if (!d_mol) {
     return "";
   }
-  return MolToMolBlock(*d_mol);
+  return MinimalLib::molblock_helper(*d_mol, details.c_str(), false);
 }
-std::string JSMol::get_v3Kmolblock() const {
+std::string JSMol::get_v3Kmolblock(const std::string &details) const {
   if (!d_mol) {
     return "";
   }
-  return MolToV3KMolBlock(*d_mol);
+  return MinimalLib::molblock_helper(*d_mol, details.c_str(), true);
 }
 std::string JSMol::get_json() const {
   if (!d_mol) {
@@ -145,7 +145,8 @@ std::string JSMol::get_pickle() const {
     return "";
   }
   std::string pickle;
-  MolPickler::pickleMol(*d_mol, pickle);
+  MolPickler::pickleMol(*d_mol, pickle,
+                        PicklerOps::AllProps ^ PicklerOps::ComputedProps);
   return pickle;
 }
 
@@ -300,6 +301,24 @@ std::string JSMol::get_atom_pair_fp_as_binary_text(
   return res;
 }
 
+std::string JSMol::get_maccs_fp() const {
+  if (!d_mol) {
+    return "";
+  }
+  auto fp = MinimalLib::maccs_fp_as_bitvect(*d_mol);
+  std::string res = BitVectToText(*fp);
+  return res;
+}
+
+std::string JSMol::get_maccs_fp_as_binary_text() const {
+  if (!d_mol) {
+    return "";
+  }
+  auto fp = MinimalLib::maccs_fp_as_bitvect(*d_mol);
+  std::string res = BitVectToBinaryText(*fp);
+  return res;
+}
+
 #ifdef RDK_BUILD_AVALON_SUPPORT
 std::string get_avalon_fp(const std::string &details) const {
   if (!d_mol) {
@@ -466,6 +485,15 @@ std::string JSMol::get_prop(const std::string &key) const {
   return val;
 }
 
+bool JSMol::clear_prop(const std::string &key) {
+  if (!d_mol) return false;
+  bool res = d_mol->hasProp(key);
+  if (res) {
+    d_mol->clearProp(key);
+  }
+  return res;
+}
+
 std::string JSMol::remove_hs() const {
   if (!d_mol) {
     return "";
@@ -480,6 +508,16 @@ std::string JSMol::remove_hs() const {
   return MolToMolBlock(molCopy, includeStereo, confId, kekulize);
 }
 
+bool JSMol::remove_hs_in_place() {
+  if (!d_mol) {
+    return false;
+  }
+
+  MolOps::removeAllHs(*d_mol);
+  MolOps::assignStereochemistry(*d_mol, true, true);
+  return true;
+}
+
 std::string JSMol::add_hs() const {
   if (!d_mol) {
     return "";
@@ -494,6 +532,17 @@ std::string JSMol::add_hs() const {
   return MolToMolBlock(molCopy, includeStereo, confId, kekulize);
 }
 
+bool JSMol::add_hs_in_place() {
+  if (!d_mol) {
+    return false;
+  }
+
+  bool addCoords = (d_mol->getNumConformers() > 0);
+  MolOps::addHs(*d_mol, false, addCoords);
+  MolOps::assignStereochemistry(*d_mol, true, true);
+  return true;
+}
+
 std::string JSMol::condense_abbreviations(double maxCoverage, bool useLinkers) {
   if (!d_mol) {
     return "";
@@ -538,6 +587,13 @@ std::string JSMol::generate_aligned_coords(const JSMol &templateMol,
                                              details.c_str());
 }
 
+int JSMol::has_coords() const {
+  if (!d_mol || !d_mol->getNumConformers()) {
+    return 0;
+  }
+  return (d_mol->getConformer().is3D() ? 3 : 2);
+}
+
 double JSMol::normalize_depiction(int canonicalize, double scaleFactor) {
   if (!d_mol || !d_mol->getNumConformers()) {
     return -1.;
@@ -552,6 +608,24 @@ void JSMol::straighten_depiction(bool minimizeRotation) {
   RDDepict::straightenDepiction(*d_mol, -1, minimizeRotation);
 }
 
+std::pair<JSMolIterator *, std::string> JSMol::get_frags(
+    const std::string &details_json) {
+  if (!d_mol) {
+    return std::make_pair(nullptr, "");
+  }
+  std::vector<int> frags;
+  std::vector<std::vector<int>> fragsMolAtomMapping;
+  bool sanitizeFrags = true;
+  bool copyConformers = true;
+  MinimalLib::get_mol_frags_details(details_json, sanitizeFrags,
+                                    copyConformers);
+  auto molFrags = MolOps::getMolFrags(*d_mol, sanitizeFrags, &frags,
+                                      &fragsMolAtomMapping, copyConformers);
+  return std::make_pair(
+      new JSMolIterator(molFrags),
+      MinimalLib::get_mol_frags_mappings(frags, fragsMolAtomMapping));
+}
+
 std::string JSReaction::get_svg(int w, int h) const {
   if (!d_rxn) {
     return "";
@@ -689,6 +763,14 @@ void prefer_coordgen(bool useCoordGen) {
 #endif
 }
 
-void use_legacy_stereo_perception(bool value) {
+bool use_legacy_stereo_perception(bool value) {
+  bool was = Chirality::getUseLegacyStereoPerception();
   Chirality::setUseLegacyStereoPerception(value);
-}
\ No newline at end of file
+  return was;
+}
+
+bool allow_non_tetrahedral_chirality(bool value) {
+  bool was = Chirality::getAllowNontetrahedralChirality();
+  Chirality::setAllowNontetrahedralChirality(value);
+  return was;
+}
