@@ -7,13 +7,12 @@
 //  which is included in the file license.txt, found at the root
 //  of the RDKit source tree.
 //
-#define CATCH_CONFIG_MAIN  // This tells Catch to provide a main() - only do
-                           // this in one cpp file
 #include "catch.hpp"
 
 #include <RDGeneral/test.h>
-#include "Conrec.h"
 #include <iostream>
+#include <fstream>
+#include "Conrec.h"
 #include <RDGeneral/Invariant.h>
 #include <RDGeneral/RDLog.h>
 #include <cmath>
