@@ -39,6 +39,14 @@ extern "C" {
 #endif
 
 #include <postgres.h>
+#ifdef PG_VERSION_NUM
+#if PG_VERSION_NUM >= 160000
+#include <varatt.h>
+#ifndef Abs
+#define Abs(x)  ((x) >= 0 ? (x) : -(x))
+#endif
+#endif
+#endif
 
 typedef bytea Mol;
 
@@ -102,7 +110,8 @@ Mol *deconstructROMol(CROMol data);
 CROMol parseMolBlob(char *data, int len);
 char *makeMolBlob(CROMol data, int *len);
 /* sanitize argument is only used if asSmarts and asQuery are false */
-CROMol parseMolText(char *data, bool asSmarts, bool warnOnFail, bool asQuery, bool sanitize);
+CROMol parseMolText(char *data, bool asSmarts, bool warnOnFail, bool asQuery,
+                    bool sanitize);
 CROMol parseMolCTAB(char *data, bool keepConformer, bool warnOnFail,
                     bool asQuery);
 char *makeMolText(CROMol data, int *len, bool asSmarts, bool cxSmiles);
@@ -118,7 +127,7 @@ bool isValidMolBlob(char *data, int len);
 
 int molcmp(CROMol i, CROMol a);
 
-int MolSubstruct(CROMol i, CROMol a, bool useChirality);
+int MolSubstruct(CROMol i, CROMol a, bool useChirality, bool useMatchers);
 int MolSubstructCount(CROMol i, CROMol a, bool uniquify, bool useChirality);
 
 bytea *makeMolSignature(CROMol data);
