@@ -490,4 +490,33 @@ select 'c1cccc1'::mol @> '[r5]'::qmol;
 select mol_in('Cc1ccc2c(c1)-n1-c(=O)c=cc(=O)-n-2-c2cc(C)ccc2-1');
 select 'c1cccc1'::text::mol;
 select 'c1cccc1'::varchar::mol;
-select mol_from_smiles('CCN(=O)=O') @> 'CN(=O)=O';
\ No newline at end of file
+select mol_from_smiles('CCN(=O)=O') @> 'CN(=O)=O';
+
+-- github #6002: molcmp failure
+select mol_cmp(mol_from_ctab('
+  Mrv2211 02092314292D
+
+  5  4  0  0  0  0            999 V2000
+    0.0000    3.6020    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
+    0.7145    4.0145    0.0000 S   0  0  0  0  0  0  0  0  0  0  0  0
+    1.4290    4.4270    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
+    1.1270    3.3001    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
+    0.3020    4.7291    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
+  1  2  6  0  0  0  0
+  2  3  2  0  0  0  0
+  2  4  2  0  0  0  0
+  2  5  6  0  0  0  0
+M  END'),mol_from_ctab('
+  Mrv2211 02092314292D
+
+  5  4  0  0  0  0            999 V2000
+    0.0000    3.6020    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
+    0.7145    4.0145    0.0000 S   0  0  0  0  0  0  0  0  0  0  0  0
+    1.4290    4.4270    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
+    1.1270    3.3001    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
+    0.3020    4.7291    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
+  1  2  6  0  0  0  0
+  2  3  2  0  0  0  0
+  2  4  2  0  0  0  0
+  2  5  6  0  0  0  0
+M  END'));
\ No newline at end of file
