@@ -7,14 +7,16 @@
 #  which is included in the file license.txt, found at the root
 #  of the RDKit source tree.
 #
-from collections import abc  # this won't work in python2, but we don't support that any more
+from collections import \
+    abc  # this won't work in python2, but we don't support that any more
 
-from rdkit import Chem
-from rdkit.Chem import rdMolDescriptors as _rdMolDescriptors
-from rdkit.Chem import rdPartialCharges, rdMolDescriptors
 import rdkit.Chem.ChemUtils.DescriptorUtilities as _du
-from rdkit.Chem.EState.EState import (MaxEStateIndex, MinEStateIndex, MaxAbsEStateIndex,
-                                      MinAbsEStateIndex)
+from rdkit import Chem
+from rdkit.Chem import rdMolDescriptors
+from rdkit.Chem import rdMolDescriptors as _rdMolDescriptors
+from rdkit.Chem import rdPartialCharges
+from rdkit.Chem.EState.EState import (MaxAbsEStateIndex, MaxEStateIndex,
+                                      MinAbsEStateIndex, MinEStateIndex)
 from rdkit.Chem.QED import qed
 
 
@@ -28,7 +30,8 @@ _descList = []
 
 def _setupDescriptors(namespace):
   global _descList, descList
-  from rdkit.Chem import GraphDescriptors, MolSurf, Lipinski, Fragments, Crippen, Descriptors3D
+  from rdkit.Chem import (Crippen, Descriptors3D, Fragments, GraphDescriptors,
+                          Lipinski, MolSurf)
   from rdkit.Chem.EState import EState_VSA
   _descList.clear()
 
@@ -268,13 +271,43 @@ class PropertyFunctor(rdMolDescriptors.PythonPropertyFunctor):
     raise NotImplementedError("Please implement the __call__ method")
 
 
+def CalcMolDescriptors(mol, missingVal=None, silent=True):
+  ''' calculate the full set of descriptors for a molecule
+    
+    Parameters
+    ----------
+    mol : RDKit molecule
+    missingVal : float, optional
+                 This will be used if a particular descriptor cannot be calculated
+    silent : bool, optional
+             if True then exception messages from descriptors will be displayed
+
+    Returns
+    -------
+    dict 
+         A dictionary with decriptor names as keys and the descriptor values as values
+    '''
+  res = {}
+  for nm, fn in _descList:
+    # some of the descriptor fucntions can throw errors if they fail, catch those here:
+    try:
+      val = fn(mol)
+    except:
+      if not silent:
+        import traceback
+        traceback.print_exc()
+      val = missingVal
+    res[nm] = val
+  return res
+
+
 # ------------------------------------
 #
 #  doctest boilerplate
 #
 def _runDoctests(verbose=None):  # pragma: nocover
-  import sys
   import doctest
+  import sys
   failed, _ = doctest.testmod(optionflags=doctest.ELLIPSIS, verbose=verbose)
   sys.exit(failed)
 
