@@ -14,13 +14,11 @@ from importlib.util import find_spec
 from io import BytesIO
 
 import numpy
-from rdkit import Chem
-from rdkit import RDConfig
-from rdkit import rdBase
+
+from rdkit import Chem, RDConfig, rdBase
 from rdkit.Chem import rdDepictor
 from rdkit.Chem.Draw import rdMolDraw2D
-from rdkit.Chem.Draw.MolDrawing import DrawingOptions
-from rdkit.Chem.Draw.MolDrawing import MolDrawing
+from rdkit.Chem.Draw.MolDrawing import DrawingOptions, MolDrawing
 from rdkit.Chem.Draw.rdMolDraw2D import *
 
 
