@@ -33,10 +33,9 @@
 import os
 import re
 import weakref
+from io import StringIO
 
-from rdkit import Chem
-from rdkit import RDConfig
-from rdkit.six.moves import cStringIO as StringIO
+from rdkit import Chem, RDConfig
 
 
 class FGHierarchyNode(object):
@@ -78,8 +77,8 @@ lastFilename = None
 
 def BuildFuncGroupHierarchy(fileNm=None, data=None, force=False):
   global groupDefns, hierarchy, lastData, lastFilename
-  if (not force and hierarchy and (not data or data == lastData) and
-      (not fileNm or fileNm == lastFilename)):
+  if (not force and hierarchy and (not data or data == lastData)
+      and (not fileNm or fileNm == lastFilename)):
     return hierarchy[:]
   lastData = data
   splitter = re.compile('\t+')
