@@ -29,6 +29,17 @@ class TestCase(unittest.TestCase):
 
     self.assertRaises(ValueError, Enumerator.EnumerateReaction, rxn, (reacts1, ))
 
+  def test_UniquifyingEnhancedStereo(self):
+    rxn = AllChem.ReactionFromSmarts('[C:1]-[O:2]>>[C:1]-[N:2]')
+    rxn.Initialize()
+    reacts1 = ['C[C@@H](F)CO', 'C[C@@H](F)CO |a:1|', 'C[C@@H](F)CO |o1:1|', 'C[C@@H](F)CO |&1:1|']
+    reacts1 = [Chem.MolFromSmiles(x) for x in reacts1]
+
+    prods = list(Enumerator.EnumerateReaction(rxn, (reacts1, ), uniqueProductsOnly=True))
+    ps = Chem.SmilesWriteParams()
+    print([Chem.MolToCXSmiles(tpl[0], ps, Chem.CXSmilesFields.CX_ENHANCEDSTEREO) for tpl in prods])
+    self.assertEqual(len(prods), len(reacts1))
+
 
 if __name__ == '__main__':  # pragma: nocover
   unittest.main()
