@@ -10,6 +10,7 @@ tests = [("python", "UnitTestChem.py", {}), ("python", "UnitTestChemv2.py", {}),
          ("python", "UnitTestFunctionalGroups.py", {}), ("python", "UnitTestCrippen.py", {}),
          ("python", "UnitTestPandasTools.py", {}), ("python", "UnitTestDocTestsChem.py", {}),
          ("python", "UnitTestFeatFinderCLI.py", {}), ("python", "UnitTestQED.py", {}),
+         ("python", "BRICS.py", {}), ("python", "EnumerateStereoisomers.py", {}),
          ("python", "UnitTestSaltRemover.py", {}), ("python", "test_list.py", {
            'dir': 'AtomPairs'
          }), ("python", "test_list.py", {
@@ -40,8 +41,7 @@ tests = [("python", "UnitTestChem.py", {}), ("python", "UnitTestChemv2.py", {}),
            'dir': 'Features'
          }), ("python", "test_list.py", {
            'dir': 'MolStandardize'
-         }), 
-         ("python", "UnitTestRegistrationHash.py", {})]
+         }), ("python", "UnitTestRegistrationHash.py", {})]
 
 # only attempt the MolKey tests if we have the pre-reqs:
 try:
@@ -58,6 +58,7 @@ longTests = [
 
 if __name__ == '__main__':
   import sys
+
   from rdkit import TestRunner
   failed, tests = TestRunner.RunScript('test_list.py', 0, 1)
   sys.exit(len(failed))
