@@ -66,39 +66,32 @@ Optional Arguments:
 """
 # from rdkit.Dbase.DbConnection import DbConnect
 
-from __future__ import print_function
-
+import pickle
 import sys
 
 import numpy
 
-from rdkit import DataStructs
-from rdkit import RDConfig
+from rdkit import DataStructs, RDConfig
 from rdkit.Dbase.DbConnection import DbConnect
 from rdkit.ML import CompositeRun
 from rdkit.ML.Data import DataUtils, SplitData, Stats
-from rdkit.six import PY3
-from rdkit.six.moves import cPickle
-from rdkit.six.moves import input
-
 
 __VERSION_STRING = "2.4.0"
 
-if PY3:
 
-  def cmp(t1, t2):
-    return (t1 < t2) * -1 or (t1 > t2) * 1
+def cmp(t1, t2):
+  return (t1 < t2) * -1 or (t1 > t2) * 1
 
 
 def message(msg, noRet=0, dest=sys.stderr):
   """ emits messages to _sys.stderr_
-    override this in modules which import this one to redirect output
+      override this in modules which import this one to redirect output
 
-    **Arguments**
+      **Arguments**
 
-      - msg: the string to be displayed
+        - msg: the string to be displayed
 
-  """
+    """
   if noRet:
     dest.write('%s ' % (msg))
   else:
@@ -107,45 +100,45 @@ def message(msg, noRet=0, dest=sys.stderr):
 
 def error(msg, dest=sys.stderr):
   """ emits messages to _sys.stderr_
-    override this in modules which import this one to redirect output
+      override this in modules which import this one to redirect output
 
-    **Arguments**
+      **Arguments**
 
-      - msg: the string to be displayed
+        - msg: the string to be displayed
 
-  """
+    """
   sys.stderr.write('ERROR: %s\n' % (msg))
 
 
 def ScreenModel(mdl, descs, data, picking=[1], indices=[], errorEstimate=0):
   """ collects the results of screening an individual composite model that match
-    a particular value
+      a particular value
 
-   **Arguments**
+     **Arguments**
 
-     - mdl: the composite model
+       - mdl: the composite model
 
-     - descs: a list of descriptor names corresponding to the data set
+       - descs: a list of descriptor names corresponding to the data set
 
-     - data: the data set, a list of points to be screened.
+       - data: the data set, a list of points to be screened.
 
-     - picking: (Optional) a list of values that are to be collected.
-       For examples, if you want an enrichment plot for picking the values
-       1 and 2, you'd having picking=[1,2].
+       - picking: (Optional) a list of values that are to be collected.
+         For examples, if you want an enrichment plot for picking the values
+         1 and 2, you'd having picking=[1,2].
 
-    **Returns**
+      **Returns**
 
-      a list of 4-tuples containing:
+        a list of 4-tuples containing:
 
-         - the id of the point
+           - the id of the point
 
-         - the true result (from the data set)
+           - the true result (from the data set)
 
-         - the predicted result
+           - the predicted result
 
-         - the confidence value for the prediction
+           - the confidence value for the prediction
 
-  """
+    """
   mdl.SetInputOrder(descs)
 
   for j in range(len(mdl)):
@@ -190,27 +183,27 @@ def ScreenModel(mdl, descs, data, picking=[1], indices=[], errorEstimate=0):
 def AccumulateCounts(predictions, thresh=0, sortIt=1):
   """  Accumulates the data for the enrichment plot for a single model
 
-    **Arguments**
+      **Arguments**
 
-      - predictions: a list of 3-tuples (as returned by _ScreenModels_)
+        - predictions: a list of 3-tuples (as returned by _ScreenModels_)
 
-      - thresh: a threshold for the confidence level.  Anything below
-        this threshold will not be considered
+        - thresh: a threshold for the confidence level.  Anything below
+          this threshold will not be considered
 
-      - sortIt: toggles sorting on confidence levels
+        - sortIt: toggles sorting on confidence levels
 
 
-    **Returns**
+      **Returns**
 
-      - a list of 3-tuples:
+        - a list of 3-tuples:
 
-        - the id of the active picked here
+          - the id of the active picked here
 
-        - num actives found so far
+          - num actives found so far
 
-        - number of picks made so far
+          - number of picks made so far
 
-  """
+    """
   if sortIt:
     predictions.sort(lambda x, y: cmp(y[3], x[3]))
   res = []
@@ -238,11 +231,11 @@ def MakePlot(details, final, counts, pickVects, nModels, nTrueActs=-1):
     if nModels > 1:
       _, sd = Stats.MeanAndDev(pickVects[i])
       confInterval = Stats.GetConfidenceInterval(sd, len(pickVects[i]), level=90)
-      outF.write('%d %f %f %d %f\n' % (i + 1, final[i][0] / counts[i], final[i][1] / counts[i],
-                                       counts[i], confInterval))
+      outF.write('%d %f %f %d %f\n' %
+                 (i + 1, final[i][0] / counts[i], final[i][1] / counts[i], counts[i], confInterval))
     else:
-      outF.write('%d %f %f %d\n' % (i + 1, final[i][0] / counts[i], final[i][1] / counts[i],
-                                    counts[i]))
+      outF.write('%d %f %f %d\n' %
+                 (i + 1, final[i][0] / counts[i], final[i][1] / counts[i], counts[i]))
     i += 1
   outF.close()
   plotFileName = '%s.gnu' % (details.plotFile)
@@ -371,7 +364,7 @@ if __name__ == '__main__':
   descs = dataSet.GetVarNames()
   nPts = dataSet.GetNPts()
   message('npts: %d\n' % (nPts))
-  final = numpy.zeros((nPts, 2), numpy.float)
+  final = numpy.zeros((nPts, 2), float)
   counts = numpy.zeros(nPts, numpy.integer)
   selPts = [None] * nPts
 
@@ -390,7 +383,7 @@ if __name__ == '__main__':
       message(' Building model %d' % len(models))
       blob = blob[0]
       try:
-        models.append(cPickle.loads(str(blob)))
+        models.append(pickle.loads(str(blob)))
       except Exception:
         import traceback
         traceback.print_exc()
@@ -405,7 +398,7 @@ if __name__ == '__main__':
   else:
     for modelName in extras:
       try:
-        model = cPickle.load(open(modelName, 'rb'))
+        model = pickle.load(open(modelName, 'rb'))
       except Exception:
         import traceback
         print('problems with model %s:' % modelName)
@@ -478,8 +471,8 @@ if __name__ == '__main__':
         print('%d\t%f\t%f\t%f\t%d\t%s' % (i + 1, final[i][0] / counts[i], confInterval,
                                           final[i][1] / counts[i], counts[i], str(selPts[i])))
       else:
-        print('%d\t%f\t%f\t%d\t%s' % (i + 1, final[i][0] / counts[i], final[i][1] / counts[i],
-                                      counts[i], str(selPts[i])))
+        print('%d\t%f\t%f\t%d\t%s' %
+              (i + 1, final[i][0] / counts[i], final[i][1] / counts[i], counts[i], str(selPts[i])))
       i += 1
 
   mean, sd = Stats.MeanAndDev(halfwayPts)
