@@ -1,9 +1,12 @@
-tests = [("python", "SimilarityPickers.py", {}), ]
+tests = [
+  ("python", "SimilarityPickers.py", {}),
+]
 
 longTests = []
 
 if __name__ == '__main__':
   import sys
+
   from rdkit import TestRunner
   failed, tests = TestRunner.RunScript('test_list.py', 0, 1)
   sys.exit(len(failed))
